const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });
const moment = require("moment");
const guadagna = require("./Guadagna.js");

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    let sentMessage;
    if (options && options.photo) {
      sentMessage = await bot.sendPhoto(chatId, options.photo, {
        caption: text,
      });
    } else {
      sentMessage = await bot.sendMessage(chatId, text, options);
    }
    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }
    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const actionStats = {};
const actionMission = {};

const keyboardBack = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [[{ text: "↩️ Torna indietro" }]],
    resize_keyboard: true,
  },
};

const keyboardMissionType = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "📋 Normale" }, { text: "⏳ Tempo" }],
      [{ text: "🔢 Numero Chiuso" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const keyboardPubblicita = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "📜 Crea Pubblicità 📜" }, { text: "📜 Attive Pubblicità 📜" }],

      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const keyboardPlatform = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🖼 Instagram" }, { text: "🟦 Facebook" }],
      [{ text: "🤖 Telegram" }, { text: "🎵 TikTok" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const keyboardActionType = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "Entra gruppo/canale" }, { text: "Mi piace" }],
      [{ text: "Segui" }, { text: "Commenta" }],
      [{ text: "Condividi" }, { text: "Visualizza" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const keyboardMissioni = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🚀 Crea Missioni 🚀" }, { text: "🚀 Cerca Missione 🚀" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const actionTypeMap = {
  "Entra gruppo/canale": "enter",
  "Mi piace": "like",
  Segui: "follow",
  Commenta: "comment",
  Condividi: "share",
  Visualizza: "view",
};

async function displayMissionPage(userMessages, chatId, page) {
  const itemPerPage = 9;
  const offset = (page - 1) * itemPerPage;
  const query = `
      SELECT m.id, m.type, m.instructions, m.link, m.status, m.user_admin
      FROM missions AS m
      WHERE m.status IN ('active', 'inactive')
      LIMIT ? OFFSET ?`;

  try {
    const [missions] = await db.promise().query(query, [itemPerPage, offset]);

    if (missions.length === 0) {
      await handleBotMessage(
        userMessages,
        chatId,
        "Nessuna missione trovata.",
        keyboardMissioni
      );
      return;
    }

    let missionButtons = [];
    for (let i = 0; i < missions.length; i += 3) {
      const row = missions.slice(i, i + 3).map((mission) => {
        const star = mission.user_admin ? " ⭐️" : "";
        return {
          text: `🚀 ID ${mission.id} - ${mission.type}${star} ${
            mission.status === "active" ? "🟢" : "🔴"
          }`,
        };
      });
      missionButtons.push(row);
    }

    let navigationButtons = [];
    if (page > 1) {
      navigationButtons.push({
        text: `⬅️ Indietro ${page - 1}`,
      });
    }
    if (missions.length === itemPerPage) {
      navigationButtons.push({
        text: `➡️ Avanti ${page + 1}`,
      });
    }

    await handleBotMessage(
      userMessages,
      chatId,
      `Le missioni con la ⭐️ vuol dire che sono create da un admin\n\nSe vuoi ricercare velocemente digita: \n🚀[spazio]ID[spazio]numero`,
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [
            ...missionButtons,
            ...navigationButtons.map((button) => [button]),
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      }
    );
  } catch (err) {
    console.error("Errore nella ricerca delle missioni:", err);
    await handleBotMessage(
      userMessages,
      chatId,
      "Si è verificato un errore durante la ricerca delle missioni.",
      keyboardMissioni
    );
  }
}

const handleGestioneMissioni = async (userMessages, chatId, userData, msg) => {
  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    actionStats[chatId] = {};
    actionMission[chatId] = {};
  }

  if (!actionMission[chatId]) {
    actionMission[chatId] = {};
  }

  if (!actionStats[chatId]) {
    actionStats[chatId] = {};
  }

  if (msg.text === "🚀 Cerca Missione 🚀") {
    await displayMissionPage(userMessages, chatId, 1);
  } else if (msg.text.startsWith("➡️ Avanti ")) {
    const nextPage = parseInt(msg.text.split(" ")[2]);
    await displayMissionPage(userMessages, chatId, nextPage);
  } else if (msg.text.startsWith("⬅️ Indietro ")) {
    const previousPage = parseInt(msg.text.split(" ")[2]);
    await displayMissionPage(userMessages, chatId, previousPage);
  }

  if (msg.text.startsWith("🚀 ID ")) {
    const missionId = parseInt(msg.text.split(" ")[2]);
    await handleBotMessage(
      userMessages,
      chatId,
      "Seleziona un azione:",
      keyboardMissioni
    );
    try {
      // Fetch the mission by its ID
      const [missions] = await db
        .promise()
        .query("SELECT * FROM `missions` WHERE id = ?", [missionId]);

      if (missions.length === 0) {
        await handleBotMessage(
          userMessages,
          chatId,
          "Missione non trovata.",
          keyboardMissioni
        );
        return;
      }

      const mission = missions[0];

      let statusIndicator;
      let statusButton;

      // Set the status indicator and the action button text based on the mission status
      switch (mission.status) {
        case "active":
          statusIndicator = "🟢 Stato: Attiva";
          statusButton = {
            text: "🔴 Inattivala",
            callback_data: `inattiva_${missionId}`,
          };
          break;
        case "inactive":
          statusIndicator = "🔴 Stato: Disattivato";
          statusButton = {
            text: "🟢 Attivala",
            callback_data: `attiva_${missionId}`,
          };
          break;
      }

      // Generate the mission message using createMissionMessage
      const missionMessage =
        (await guadagna.createMissionMessage(mission).text) +
        `\n\n${statusIndicator}`;

      // Define the base inline keyboard
      const inlineKeyboard = {
        inline_keyboard: [
          [
            statusButton, // Dynamically set based on mission status
          ],
          [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
        ],
      };

      // Conditionally add the "Disapprova" button if mission.user_admin is not 1
      if (mission.user_admin !== 1) {
        inlineKeyboard.inline_keyboard[0].push({
          text: "❌ Disapprova",
          callback_data: `disapprova_${missionId}`,
        });
      }

      // Send the mission message with the inline buttons
      await handleBotMessage(userMessages, chatId, missionMessage, {
        parse_mode: "HTML",
        reply_markup: inlineKeyboard,
      });
    } catch (err) {
      console.error("Errore nel recupero della missione:", err);
      await handleBotMessage(
        userMessages,
        chatId,
        "Si è verificato un errore durante il recupero della missione.",
        keyboardMissioni
      );
    }
  }

  if (msg.text === "🚀 Crea Missioni 🚀") {
    actionMission[chatId] = { step: 1 };
    await handleBotMessage(
      userMessages,
      chatId,
      "Seleziona il tipo di missione:",
      keyboardMissionType
    );
  } else if (actionMission[chatId].step === 1) {
    if (msg.text === "📋 Normale") {
      actionMission[chatId] = { type: "normal", step: 2 };
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci i punti:",
        keyboardBack
      );
    } else if (msg.text === "⏳ Tempo") {
      actionMission[chatId] = { type: "time", step: 2 };
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci le istruzioni:",
        keyboardBack
      );
    } else if (msg.text === "🔢 Numero Chiuso") {
      actionMission[chatId] = { type: "close_number", step: 2 };
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci le istruzioni:",
        keyboardBack
      );
    }
  }

  // Step 2: Input for 'normal' type
  else if (
    actionMission[chatId].type === "normal" &&
    actionMission[chatId].step === 2
  ) {
    actionMission[chatId].points = parseInt(msg.text.trim(), 10);
    actionMission[chatId].step = 3;
    await handleBotMessage(
      userMessages,
      chatId,
      "Seleziona la piattaforma:",
      keyboardPlatform
    );
  } else if (
    actionMission[chatId].type === "normal" &&
    actionMission[chatId].step === 3
  ) {
    actionMission[chatId].platform = msg.text
      .split(" ")[1]
      .trim()
      .toLowerCase();
    actionMission[chatId].step = 4;
    await handleBotMessage(
      userMessages,
      chatId,
      "Inserisci il tipo di azione:",
      keyboardActionType
    );
  } else if (
    actionMission[chatId].type === "normal" &&
    actionMission[chatId].step === 4
  ) {
    actionMission[chatId].action_type = actionTypeMap[msg.text.trim()];
    actionMission[chatId].step = 5;
    await handleBotMessage(
      userMessages,
      chatId,
      "Inserisci il link:",
      keyboardBack
    );
  } else if (
    actionMission[chatId].type === "normal" &&
    actionMission[chatId].step === 5
  ) {
    actionMission[chatId].link = msg.text.trim();

    const { points, action_type, link, platform } = actionMission[chatId];

    const query = `
      INSERT INTO missions (type, platform, points, action_type, status, user_admin, user_id, link)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?)`;

    try {
      await db.promise().query(query, [
        "normal", // type
        platform, // platform
        points, // points
        action_type, // action_type
        "active", // status
        1, // user_admin
        userData.id, // user_id
        link, // link
      ]);

      await handleBotMessage(
        userMessages,
        chatId,
        "Missione normale creata con successo!",
        keyboardMissioni
      );
    } catch (err) {
      console.error("Errore nell'inserimento della missione:", err);
      await handleBotMessage(
        userMessages,
        chatId,
        "Si è verificato un errore durante la creazione della missione."
      );
    }

    delete actionMission[chatId];
  }
  // Step 2: Input for 'time' type
  else if (actionMission[chatId].type === "time") {
    if (actionMission[chatId].step === 2) {
      actionMission[chatId].instructions = msg.text.trim();
      actionMission[chatId].step = 3;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci i punti:",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 3) {
      actionMission[chatId].points = parseInt(msg.text.trim(), 10);
      actionMission[chatId].step = 4;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci il link:",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 4) {
      actionMission[chatId].link = msg.text.trim();
      actionMission[chatId].step = 5;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci la data di fine (formato DD-MM-YYYY HH:MM):",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 5) {
      const endDate = msg.text.trim();
      const dateRegex = /^\d{2}-\d{2}-\d{4} \d{2}:\d{2}$/;
      if (!dateRegex.test(endDate)) {
        await handleBotMessage(
          userMessages,
          chatId,
          "Il formato non è valido. Inserisci la data di fine nel formato corretto (DD-MM-YYYY HH:MM):",
          keyboardBack
        );
        return;
      }

      const [datePart, timePart] = endDate.split(" ");
      const [day, month, year] = datePart.split("-");
      const formattedEndDate = `${year}-${month}-${day} ${timePart}`;

      actionMission[chatId].endDate = formattedEndDate;
      const { instructions, points, link } = actionMission[chatId];

      const query = `
        INSERT INTO missions (type, platform, points, instructions, link, end, status, user_admin, user_id)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)`;

      try {
        await db.promise().query(query, [
          "time", // type
          null, // platform
          points, // points
          instructions, // instructions
          link, // link
          formattedEndDate, // end
          "active", // status
          1, // user_admin
          userData.id, // user_id
        ]);

        await handleBotMessage(
          userMessages,
          chatId,
          "Missione a tempo creata con successo!",
          keyboardMissioni
        );
      } catch (err) {
        console.error("Errore nell'inserimento della missione:", err);
        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore durante la creazione della missione."
        );
      }

      delete actionMission[chatId];
    }
  }

  // Step 2: Input for 'close_number' type
  else if (actionMission[chatId].type === "close_number") {
    if (actionMission[chatId].step === 2) {
      actionMission[chatId].instructions = msg.text.trim();
      actionMission[chatId].step = 3;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci il numero massimo di utenti:",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 3) {
      actionMission[chatId].max_user = parseInt(msg.text.trim(), 10);
      actionMission[chatId].step = 4;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci i punti:",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 4) {
      actionMission[chatId].points = parseInt(msg.text.trim(), 10);
      actionMission[chatId].step = 5;
      await handleBotMessage(
        userMessages,
        chatId,
        "Inserisci il link:",
        keyboardBack
      );
    } else if (actionMission[chatId].step === 5) {
      actionMission[chatId].link = msg.text.trim();
      const { instructions, max_user, points, link } = actionMission[chatId];

      const query = `
        INSERT INTO missions (type, platform, points, instructions, link, max_user,count_user,  status, user_admin, user_id)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?,?)`;

      try {
        await db.promise().query(query, [
          "close_number", // type
          null, // platform
          points, // points
          instructions, // instructions
          link, // link
          max_user, // max_user
          0, // count_user
          "active", // status
          1, // user_admin
          userData.id, // user_id
        ]);

        await handleBotMessage(
          userMessages,
          chatId,
          "Missione a numero chiuso creata con successo!",
          keyboardMissioni
        );
      } catch (err) {
        console.error("Errore nell'inserimento della missione:", err);
        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore durante la creazione della missione."
        );
      }

      delete actionMission[chatId];
    }
  }

  if (msg.text === "📜 Pubblicità 📜") {
    await handleBotMessage(
      userMessages,
      chatId,
      "Seleziona un azione per proseguire.",
      keyboardPubblicita
    );
  }

  if (msg.text === "🚀 Missioni 🚀") {
    await handleBotMessage(
      userMessages,
      chatId,
      "Seleziona un azione per proseguire.",
      keyboardMissioni
    );
  }

  if (msg.text === "📜 Crea Pubblicità 📜") {
    actionStats[chatId] = { step: 1 };
    await handleBotMessage(
      userMessages,
      chatId,
      "Inserisci la descrizione della pubblicità:",
      keyboardBack
    );
  } else if (actionStats[chatId].step === 1) {
    actionStats[chatId].description = msg.text;
    actionStats[chatId].step = 2;
    await handleBotMessage(
      userMessages,
      chatId,
      "Inserisci il link della pubblicità:",
      keyboardBack
    );
  } else if (actionStats[chatId].step === 2) {
    actionStats[chatId].link = msg.text;
    actionStats[chatId].step = 3;
    await handleBotMessage(
      userMessages,
      chatId,
      "Inserisci la data di fine (formato DD-MM-YYYY HH:MM):",
      keyboardBack
    );
  } else if (actionStats[chatId].step === 3) {
    const endDate = msg.text.trim();

    // Validate the input format 'DD-MM-YYYY HH:MM'
    const dateRegex = /^\d{2}-\d{2}-\d{4} \d{2}:\d{2}$/;
    if (!dateRegex.test(endDate)) {
      await handleBotMessage(
        userMessages,
        chatId,
        "Il formato non è valido. Inserisci la data di fine nel formato corretto (DD-MM-YYYY HH:MM):",
        keyboardBack
      );
      return;
    }

    // Convert the date to 'YYYY-MM-DD HH:MM' format
    const [datePart, timePart] = endDate.split(" ");
    const [day, month, year] = datePart.split("-");
    const formattedEndDate = `${year}-${month}-${day} ${timePart}`;

    actionStats[chatId].endDate = formattedEndDate;

    const { description, link } = actionStats[chatId];

    // Insert into the `missions` table
    const query = `
      INSERT INTO missions (type, platform, points, status, user_admin, user_id, instructions, link, end)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    try {
      await db.promise().query(query, [
        "advertising", // type
        null, // platform
        0, // points
        "active", // status
        1, // user_admin
        userData.id, // user_id
        description, // instructions
        link, // link
        formattedEndDate, // end
      ]);

      await handleBotMessage(
        userMessages,
        chatId,
        "Pubblicità creata con successo!",
        keyboardPubblicita
      );
    } catch (err) {
      console.error("Errore nell'inserimento della pubblicità:", err);
      await handleBotMessage(
        userMessages,
        chatId,
        "Si è verificato un errore durante la creazione della pubblicità.",
        keyboardBack
      );
    }

    // Reset the action for the chat
    delete actionStats[chatId];
  }

  if (msg.text === "📜 Attive Pubblicità 📜") {
    const query = `SELECT  
    m.id AS mission_id,
    m.instructions,
    m.link,
    m.start,
    m.end,COUNT(ms.status) AS completed_users FROM  missions AS m LEFT JOIN mission_status AS ms 
    ON m.id = ms.mission_id 
    AND ms.status = 'complete'
WHERE m.type = 'advertising' AND m.status = 'active'
GROUP BY  m.id, m.instructions, m.link, m.start, m.end`;
    try {
      const [rows] = await db.promise().query(query);
      if (rows.length === 0) {
        await handleBotMessage(
          userMessages,
          chatId,
          "Non ci sono pubblicità attive.",
          keyboardPubblicita
        );
        return;
      }

      for (const mission of rows) {
        const text =
          `🚀 PUBBLICITA🚀 \n\n` +
          `${mission.instructions}\n\n` +
          `Visita il seguente link ⬇️\n` +
          `📍 Link: ${mission.link}\n\n` +
          `📅 Inizio: ${moment(mission.start).format("DD/MM/YYYY HH:mm")}\n` +
          `📅 Fine: ${moment(mission.end).format("DD/MM/YYYY HH:mm")}\n\n` +
          `👥 Visualizzazioni: ${mission.completed_users}`;

        const options = {
          parse_mode: "HTML",
          reply_markup: {
            inline_keyboard: [
              [
                {
                  text: "❌ Disattiva",
                  callback_data: `deactivate-${mission.id}`,
                },
              ],
            ],
          },
        };

        await handleBotMessage(userMessages, chatId, text, options);
      }

      await handleBotMessage(
        userMessages,
        chatId,
        "Seleziona quale pubblicità vuoi disattivare in anticipo",
        keyboardPubblicita
      );
    } catch (err) {
      console.error("Errore nel recupero delle pubblicità attive:", err);
      await handleBotMessage(
        userMessages,
        chatId,
        "Si è verificato un errore durante il recupero delle pubblicità attive."
      );
    }
  }
};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  if (msg.data.startsWith("disapprova_")) {
    const [action, missionId] = msg.data.split("_");

    if (action === "disapprova") {
      const query =
        "UPDATE missions SET status = 'pending', approved = 0 WHERE id = ?";
      try {
        await db.promise().query(query, [missionId]);

        await handleBotMessage(
          userMessages,
          chatId,
          "La Missione è stata disapprovata.",
          keyboardMissioni
        );
      } catch (err) {
        console.error("Errore nella disapprovazione:", err);

        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore nella disapprovazione della missione.",
          keyboardMissioni
        );
      }
    }
  }
  if (msg.data.startsWith("attiva_")) {
    const [action, missionId] = msg.data.split("_");

    if (action === "attiva") {
      const query = "UPDATE missions SET status = 'active' WHERE id = ?";
      try {
        await db.promise().query(query, [missionId]);

        await handleBotMessage(
          userMessages,
          chatId,
          "La Missione è stata riattivata.",
          keyboardMissioni
        );
      } catch (err) {
        console.error("Errore nella attivazione:", err);

        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore nella attivazione della missione.",
          keyboardMissioni
        );
      }
    }
  }

  if (msg.data.startsWith("inattiva_")) {
    const [action, missionId] = msg.data.split("_");

    if (action === "inattiva") {
      const query = "UPDATE missions SET status = 'inactive' WHERE id = ?";
      try {
        await db.promise().query(query, [missionId]);

        await handleBotMessage(
          userMessages,
          chatId,
          "La Missione è stata disattivata.",
          keyboardMissioni
        );
      } catch (err) {
        console.error("Errore nella disattivazione della pubblicità:", err);

        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore nella disattivazione della pubblicità.",
          keyboardMissioni
        );
      }
    }
  }
  if (msg.data.startsWith("deactivate-")) {
    const [action, missionId] = msg.data.split("-");

    if (action === "deactivate") {
      const query = "UPDATE missions SET status = 'inactive' WHERE id = ?";
      try {
        await db.promise().query(query, [missionId]);

        await handleBotMessage(
          userMessages,
          chatId,
          "La pubblicità è stata disattivata.",
          keyboardPubblicita
        );
      } catch (err) {
        console.error("Errore nella disattivazione della pubblicità:", err);

        await handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore nella disattivazione della pubblicità.",
          keyboardPubblicita
        );
      }
    }
  }
};

module.exports = {
  handleGestioneMissioni,
  handleCallbackQuery,
};
