const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const keyboardGestione = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🟢 Missioni Attive" }, { text: "🔴 Missioni Inattive" }],
      [{ text: "🟡 Missioni Messe in Pausa" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};
const handlePost = async (userMessages, chatId, userData, msg) => {
  if (msg.text === "🗒 GESTIONE DEI POST") {
    await handleBotMessage(
      userMessages,
      chatId,
      "In questa sezione puoi amministrare i tuoi annunci: metterli in pausa, modificarli o eliminarli.\nOltre a continuare le missioni che hai messo in pausa",
      keyboardGestione
    );
  }

  if (msg.text === "🟢 Missioni Attive") {
    handleActiveInactiveMissions(userMessages, chatId, userData.id, "active");
  }

  if (msg.text === "🔴 Missioni Inattive") {
    handleActiveInactiveMissions(userMessages, chatId, userData.id, "inactive");
  }

  handleUserMessage(userMessages, chatId, userData, msg);
};

const handleActiveInactiveMissions = async (
  userMessages,
  chatId,
  userId,
  status,
  page = 1
) => {
  const itemsPerPage = 1;
  const offset = (page - 1) * itemsPerPage;

  db.query(
    `SELECT id, link, platform, points, action_type 
           FROM missions 
           WHERE user_id = ? AND status = ? 
           LIMIT ? OFFSET ?`,
    [userId, status, itemsPerPage, offset],
    async (err, results) => {
      if (err) {
        console.error("Errore nel recupero delle missioni:", err);
        return;
      }

      if (results.length === 0) {
        await handleBotMessage(
          userMessages,
          chatId,
          "Nessuna missione trovata.",
          keyboardGestione
        );
        return;
      }

      const mission = results[0];
      const text = `🚀 ID Missione: ${mission.id}\n\n⭐️ Missione Social: ${
        mission.platform
      } ⭐️\n👉🏻 ${mission.action_type} un utente/gruppo\n📍 Link: ${
        mission.link
      }\n\n${status === "active" ? "🟢 Attiva" : "🔴 Inattiva"}\n\n💰 Punti: ${
        mission.points
      }`;

      let inlineKeyboard = [
        [
          {
            text: "📍 Modifica Link",
            callback_data: `edit_link_${mission.id}_${page}`,
          },
          {
            text: "💰 Modifica Punti",
            callback_data: `edit_points_${mission.id}_${page}`,
          },
        ],
      ];

      if (status === "inactive") {
        inlineKeyboard = [
          [
            {
              text: "🔵 Attivala",
              callback_data: `active_mission_${mission.id}`,
            },
          ],
        ];
      } else {
        inlineKeyboard.push([
          {
            text: "🔴 Disattiva",
            callback_data: `inactive_mission_${mission.id}`,
          },
        ]);
      }

      const navigationButtons = [];
      if (page > 1) {
        navigationButtons.push({
          text: "⬅️ Precedente",
          callback_data: `prev_mission_${status}_${page - 1}`,
        });
      }

      navigationButtons.push({
        text: "➡️ Successivo",
        callback_data: `next_mission_${status}_${page + 1}`,
      });

      if (navigationButtons.length > 0) {
        inlineKeyboard.push(navigationButtons);
      }

      inlineKeyboard.push([
        { text: "↩️ Torna indietro", callback_data: "annulla" },
      ]);

      const options = {
        reply_markup: {
          inline_keyboard: inlineKeyboard,
        },
      };

      await handleBotMessage(userMessages, chatId, text, options);
    }
  );
};

// Stato per memorizzare le azioni richieste per ciascun utente
let userActions = {};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  const data = msg.data;
  const userId = userData.id;

  if (data.startsWith("prev_mission_") || data.startsWith("next_mission_")) {
    const [action, actionMStr, status, pageStr] = data.split("_");

    const page = parseInt(pageStr);

    if (page > 0 || data.startsWith("next_mission_")) {
      await handleActiveInactiveMissions(
        userMessages,
        chatId,
        userId,
        status,
        page
      );
    }
  }

  if (data.startsWith("edit_link_")) {
    const missionId = parseInt(data.split("_")[2]);
    const page = parseInt(data.split("_")[3] || "1"); // Usa la pagina attuale
    await handleBotMessage(
      userMessages,
      chatId,
      `Invia ora il nuovo link per la missione ID ${missionId}.`
    );

    // Salviamo l'azione dell'utente e la pagina corrente
    userActions[chatId] = { action: "edit_link", missionId, page };
  }

  if (data.startsWith("edit_points_")) {
    const missionId = parseInt(data.split("_")[2]);
    const page = parseInt(data.split("_")[3] || "1"); // Usa la pagina attuale

    db.query(
      `SELECT platform, action_type FROM missions WHERE id = ? AND user_id = ?`,
      [missionId, userId],
      async (err, results) => {
        if (err) {
          console.error("Errore nel recupero della missione:", err);
          return;
        }

        const mission = results[0];
        db.query(
          `SELECT points_min, points_max FROM platform_action_points_min_max WHERE platform = ? AND action_type = ?`,
          [mission.platform, mission.action_type],
          async (err, results) => {
            if (err) {
              console.error("Errore nel recupero dei limiti di punti:", err);
              return;
            }

            const { points_min, points_max } = results[0];
            await handleBotMessage(
              userMessages,
              chatId,
              `Inserisci un nuovo valore per i punti (tra ${points_min} e ${points_max}).`
            );

            // Salviamo l'azione dell'utente insieme ai limiti e la pagina corrente
            userActions[chatId] = {
              action: "edit_points",
              missionId,
              points_min,
              points_max,
              page,
            };
          }
        );
      }
    );
  }

  if (
    data.startsWith("inactive_mission_") ||
    data.startsWith("active_mission_")
  ) {
    const [status, missionStr, missionStrId] = data.split("_");

    const missionId = parseInt(missionStrId);

    // Aggiorna lo status della missione a "inactive"
    db.query(
      `UPDATE missions SET status = ? WHERE id = ? AND user_id = ?`,
      [status, missionId, userId],
      async (err) => {
        if (err) {
          console.error("Errore durante l'aggiornamento dello status:", err);
          return;
        }

        await handleBotMessage(
          userMessages,
          chatId,
          `La missione ID ${missionId} è stata ${
            status === "active" ? "attivata" : "disattivata"
          }.`
        );

        handleActiveInactiveMissions(
          userMessages,
          chatId,
          userData.id,
          status === "active" ? "inactive" : "active"
        );
      }
    );
  }
};

// Gestore per i messaggi successivi (dopo che l'azione è stata salvata)
const handleUserMessage = async (userMessages, chatId, userData, msg) => {
  const text = msg.text;

  // Verifica se c'è un'azione in sospeso per l'utente
  if (userActions[chatId]) {
    const { action, missionId, points_min, points_max, page } =
      userActions[chatId];

    if (action === "edit_link") {
      // Aggiorniamo il link
      db.query(
        `UPDATE missions SET link = ? WHERE id = ?`,
        [text, missionId],
        (err) => {
          if (err) {
            console.error("Errore durante l'aggiornamento del link:", err);
            return;
          }
          handleBotMessage(
            userMessages,
            chatId,
            `Il link per la missione ID ${missionId} è stato aggiornato.`
          );
          handleActiveInactiveMissions(
            userMessages,
            chatId,
            userData.id,
            "active",
            page
          );
        }
      );
      delete userActions[chatId];
    }

    if (action === "edit_points") {
      const newPoints = parseInt(text);

      if (newPoints >= points_min && newPoints <= points_max) {
        // Aggiorniamo i punti
        db.query(
          `UPDATE missions SET points = ? WHERE id = ?`,
          [newPoints, missionId],
          (err) => {
            if (err) {
              console.error("Errore durante l'aggiornamento dei punti:", err);
              return;
            }
            handleBotMessage(
              userMessages,
              chatId,
              `I punti per la missione ID ${missionId} sono stati aggiornati a ${newPoints}.`
            );
            handleActiveInactiveMissions(
              userMessages,
              chatId,
              userData.id,
              "active",
              page
            );
          }
        );
        // Puliamo lo stato dell'utente una volta completata l'azione
        delete userActions[chatId];
      } else {
        handleBotMessage(
          userMessages,
          chatId,
          `Il valore inserito non è valido. Inserisci un numero tra ${points_min} e ${points_max}.`
        );
      }
    }
  }
};

module.exports = {
  handlePost,
  handleCallbackQuery,
};
