const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});
const userStates = {}; // Oggetto per tenere traccia dello stato degli utenti (es. piattaforma)
const userMissionStatus = {};
const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const saveUserProfile = (telegramId, platform, username) => {
  return new Promise((resolve, reject) => {
    const column =
      platform === "instagram"
        ? "instagram"
        : platform === "facebook"
        ? "facebook"
        : platform === "tiktok"
        ? "tiktok"
        : "telegram";
    const query = `UPDATE telegram_users SET ${column} = ? WHERE telegram_id = ?`;
    db.query(query, [username, telegramId], (err, result) => {
      if (err) return reject(err);
      resolve(result);
    });
  });
};

const updateMissionStatus = (
  missionId,
  userId,
  status = null,
  textAdded = null,
  attached = null
) => {
  return new Promise((resolve, reject) => {
    if (status !== null) {
      // Esegui l'inserimento se lo status è fornito
      const query = `
        INSERT INTO mission_status (mission_id, user_id, status, ranking_points, text_added, attached)
        SELECT 
          m.id,
          ? AS user_id,
          ? AS status,
          IFNULL((SELECT pap.points FROM platform_ranking_action_points pap WHERE m.action_type = pap.action_type LIMIT 1), 0) AS ranking_points,
          ? AS text_added,
          ? AS attached
        FROM missions m
        WHERE m.id = ?
        ON DUPLICATE KEY UPDATE
          status = VALUES(status),
          ranking_points = VALUES(ranking_points),
          text_added = VALUES(text_added),
          attached = VALUES(attached);`;

      db.query(
        query,
        [userId, status, textAdded, attached, missionId],
        (err, result) => {
          if (err) return reject(err);
          resolve(result);
        }
      );
    } else {
      // Esegui l'aggiornamento se lo status non è fornito
      let fieldsToUpdate = [];
      let values = [];

      if (textAdded !== null) {
        fieldsToUpdate.push("text_added = ?");
        values.push(textAdded);
      }

      if (attached !== null) {
        fieldsToUpdate.push("attached = ?");
        values.push(attached);
      }

      if (fieldsToUpdate.length === 0) {
        return reject(new Error("No fields provided to update."));
      }

      const query = `
        UPDATE mission_status
        SET 
          ${fieldsToUpdate.join(", ")}
        WHERE 
          mission_id = ? AND user_id = ?;`;

      db.query(query, [...values, missionId, userId], (err, result) => {
        if (err) return reject(err);
        resolve(result);
      });
    }
  });
};

// Funzione per gestire la logica delle missioni
const handleAction = async (userMessages, chatId, platform, userData) => {
  if (userStates[chatId] === "paused") {
    return handlePausedMissions(userMessages, chatId, userData);
  }
  userStates[chatId] = platform;

  try {
    const platformValues =
      Array.isArray(platform) && platform.length > 0 ? platform : [null];

    // Costruisci la parte della query
    const platformCondition =
      Array.isArray(platform) && platform.length > 0
        ? `(m.platform IN (${platform
            .map(() => "?")
            .join(", ")}) OR (m.platform IS NULL AND m.action_type IS NULL))`
        : "(m.platform IS NULL AND m.action_type IS NULL)";

    // Query per le missioni attive
    const query = `
      SELECT m.*
      FROM missions m
      LEFT JOIN mission_status ms ON m.id = ms.mission_id AND ms.user_id = ?
      WHERE ms.mission_id IS NULL
        AND m.status = 'active'
        AND (${platformCondition})
        AND (m.start <= NOW() AND (m.end IS NULL OR m.end >= NOW()))
      ORDER BY m.user_admin DESC, m.user_vip DESC, m.points DESC
      LIMIT 1;
    `;

    const parameters = [
      userData.id,
      ...platformValues,
      platformValues[0], // Aggiungi il valore di fallback per il caso in cui `platform` sia null
    ];

    // Esegui la query per le missioni attive
    const [missions] = await db.promise().query(query, parameters);

    if (missions?.length === 0) {
      await handleBotMessage(
        userMessages,
        chatId,
        `😔 Mi dispiace, non sono presenti altre campagne !`,
        {
          reply_markup: {
            keyboard: [[{ text: "↩️ Torna indietro" }]],
            resize_keyboard: true,
          },
        }
      );
      return;
    }

    const mission = missions[0];
    const { text, options } = createMissionMessage(mission);

    await handleBotMessage(userMessages, chatId, "⭐️ Socialife:", {
      reply_markup: {
        keyboard: [[{ text: "↩️ Torna indietro" }]],
        resize_keyboard: true,
      },
    });

    await handleBotMessage(userMessages, chatId, text, options);
  } catch (err) {
    console.error("Errore nella gestione dell'azione:", err);
  }
};

const handlePausedMissions = async (userMessages, chatId, userData) => {
  userStates[chatId] = "paused";
  try {
    const query = `
      SELECT m.*
      FROM missions m
      JOIN mission_status ms ON m.id = ms.mission_id
      WHERE ms.user_id = ?
        AND ms.status = 'standby'
        AND m.status = 'active'
        AND (m.start <= NOW() AND (m.end IS NULL OR m.end >= NOW()))
      ORDER BY m.user_admin DESC, m.user_vip DESC, m.points DESC
      LIMIT 1;
    `;

    const parameters = [userData.id];

    const [missions] = await db.promise().query(query, parameters);

    if (missions?.length === 0) {
      await handleBotMessage(
        userMessages,
        chatId,
        `😔 Non hai missioni in pausa al momento.`,
        {
          reply_markup: {
            keyboard: [[{ text: "↩️ Torna indietro" }]],
            resize_keyboard: true,
          },
        }
      );
      return;
    }

    const mission = missions[0];
    const { text, options } = createMissionMessage(mission);

    await handleBotMessage(userMessages, chatId, text, options);
  } catch (err) {
    console.error("Errore nella gestione delle missioni in pausa:", err);
  }
};

const translateAction = (action) => {
  switch (action) {
    case "enter":
      return "entra";
    case "like":
      return "mi piace";
    case "follow":
      return "segui";
    case "comment":
      return "commenta";
    case "share":
      return "condividi";
    case "view":
      return "visualizza";
    default:
      return "azione sconosciuta";
  }
};

const createMissionMessage = (mission) => {
  let text = "";
  if (mission.type === "time") {
    const endDate = new Date(mission.end);
    const day = endDate.getDate().toString().padStart(2, "0");
    const month = (endDate.getMonth() + 1).toString().padStart(2, "0");
    const year = endDate.getFullYear();
    const hours = endDate.getHours().toString().padStart(2, "0");
    const minutes = endDate.getMinutes().toString().padStart(2, "0");

    text =
      `🚀 Missione a Tempo 🚀\n\n` +
      `ℹ️ <b>Istruzioni:</b>\n${mission.instructions}\n\n` +
      `⌛️ Scade il ${day}/${month}/${year} ore ${hours}:${minutes}\n` +
      `💰 Guadagnerai: ${mission.points} punti.\n\n` +
      `Visita il seguente link ⬇️\n` +
      `📍 Link: ${mission.link}\n\n` +
      `⚠️ Missione a Tempo! Completa la missione prima di tutti! I punti verranno accreditati non appena la missione verrà portata a termine con successo e sarà convalidata dall’Azienda Partner.\n\n⚠️ Se non sei riuscito a completare la missione, non preoccuparti! Metti in Pausa la missione e la riprendi successivamente.`;
  } else if (mission.type === "close_number") {
    text =
      `🚀 Missione a Numero Chiuso 🚀 \n\n` +
      `ℹ️ <b>Istruzioni:</b>\n${mission.instructions}\n\n` +
      `👥 ${mission.count_user} di ${mission.max_user}\n` +
      `💰 Guadagnerai: ${mission.points} punti.\n\n` +
      `Visita il seguente link ⬇️\n` +
      `📍 Link: ${mission.link}\n\n` +
      `⚠️ Missione a numero chiuso! Completa la missione prima di tutti! I punti verranno accreditati non appena la missione verrà portata a termine con successo e sarà convalidata dall’Azienda Partner.\n\n⚠️ Se non sei riuscito a completare la missione, non preoccuparti! Metti in Pausa la missione e la riprendi successivamente.`;
  } else if (mission.type === "advertising") {
    text =
      `🚀 PUBBLICITA🚀 \n\n` +
      `${mission.instructions}\n\n` +
      `Visita il seguente link ⬇️\n` +
      `📍 Link: ${mission.link}\n\n` +
      `⚠️ Socialife non è responsabile per il contenuto, l’accuratezza o la legalità delle pubblicità pubblicate sulla piattaforma. In caso di frodi, pratiche ingannevoli o qualsiasi altro comportamento illecito legato alle pubblicità, Socialife declina ogni responsabilità. Gli inserzionisti sono gli unici responsabili per le informazioni fornite e per eventuali conseguenze derivanti dalla loro pubblicazione.`;
  } else {
    text =
      `⭐️ <b>Missione Social: ${
        mission.platform.charAt(0).toUpperCase() + mission.platform.slice(1)
      }</b> ⭐️\n\n` +
      `👉🏻 ${translateAction(mission.action_type)} un utente\n` +
      `📍 Link: ${mission.link}\n\n` +
      `💰 Guadagnerai: ${mission.points} punti.\n\n` +
      `⚠️  Una volta completata la missione, non rimuovere il ${translateAction(
        mission.action_type
      )}.`;
  }

  let options = {
    parse_mode: "HTML",
    reply_markup: {
      inline_keyboard: [
        [
          { text: "✅ Conferma", callback_data: "gua-con-" + mission.id },
          {
            text: "⏩ Salta",
            callback_data: "gua-skip-" + mission.id,
          },
        ],
        [
          {
            text: "❗️ Ho avuto un problema.",
            callback_data: "gua-prob-" + mission.id,
          },
        ],
      ],
      resize_keyboard: true,
    },
  };
  if (mission.type === "time" || mission.type === "close_number") {
    options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: [
          [
            { text: "✅ Conferma", callback_data: "gua-con-" + mission.id },
            {
              text: "🟡 Metti in Pausa",
              callback_data: "gua-sal-" + mission.id,
            },
          ],
          [
            {
              text: "⏩ Salta",
              callback_data: "gua-skip-" + mission.id,
            },
            {
              text: "❗️ Ho avuto un problema.",
              callback_data: "gua-prob-" + mission.id,
            },
          ],
        ],
        resize_keyboard: true,
      },
    };
  }

  if (mission.type === "advertising") {
    const inlineKeyboard = [
      [
        { text: "➡️ Salta", callback_data: "gua-pubb-" + mission.id },
        {
          text: "❗️ Segnala Pubblicità.",
          callback_data: "gua-prob-" + mission.id,
        },
      ],
      [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
    ];

    if (mission.link) {
      inlineKeyboard.unshift([
        { text: "➡️ Visita lo Sponsor ⬅️", url: mission.link },
      ]);
    }

    options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: inlineKeyboard,
        resize_keyboard: true,
      },
    };
  }

  return { text, options };
};

const keyboardGuadagna = [
  [{ text: "🚀 Tutte le Missioni 🚀" }],
  [{ text: "Instagram" }, { text: "Facebook" }],
  [{ text: "TikTok" }, { text: "Telegram" }],
  [{ text: "↩️ Torna indietro" }],
];
// Funzione per gestire Guadagna
const handleGuadagna = async (userMessages, chatId, userData, msg) => {
  if (!userData) {
    return;
  }

  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    userStates[chatId] = null; // Resetta lo stato dell'utente
    userMissionStatus[chatId] = null;
  }

  if (msg.text === "🟡 Missioni Messe in Pausa") {
    await handlePausedMissions(userMessages, chatId, userData);
  }

  if (msg.text === "🚀 Tutte le Missioni 🚀") {
    const platforms = ["telegram"];

    if (userData.instagram) {
      platforms.push("instagram");
    }

    if (userData.facebook) {
      platforms.push("facebook");
    }

    if (userData.tiktok) {
      platforms.push("tiktok");
    }

    await handleAction(userMessages, chatId, platforms, userData);
  }

  // Logica per la selezione della piattaforma
  if (msg.text === "💶 GUADAGNA 💶") {
    userStates[chatId] = null; // Resetta lo stato dell'utente
    const options = {
      reply_markup: {
        keyboard: keyboardGuadagna,
        resize_keyboard: true,
      },
    };

    await handleBotMessage(
      userMessages,
      chatId,
      "Benvenuto nella tua area personale. Seleziona una delle opzioni:",
      options
    );
  } else if (
    ["Instagram", "Facebook", "TikTok", "Telegram"].includes(msg.text)
  ) {
    userStates[chatId] = msg.text.toLowerCase();

    const platform = userStates[chatId];
    if (!userData[platform] && !msg.text === "Telegram") {
      const text = `Inserisci il tuo username di ${msg.text}.`;
      const options = {
        reply_markup: {
          keyboard: [
            [{ text: "❌ Non ho un profilo " + msg.text + " Idoneo ❌" }],
          ],
          resize_keyboard: true,
        },
      };

      await handleBotMessage(userMessages, chatId, text, options);
    } else {
      await handleAction(userMessages, chatId, [platform], userData);
    }
  } else if (userStates[chatId]) {
    const platform = userStates[chatId];

    if (["instagram", "facebook", "tiktok", "telegram"].includes(platform)) {
      const username = msg.text;

      try {
        if (msg.text.startsWith("❌ Non ho un profilo")) {
          const options = {
            reply_markup: {
              keyboard: [[{ text: "↩️ Torna indietro" }]],
              resize_keyboard: true,
            },
          };

          await handleBotMessage(
            userMessages,
            chatId,
            `⚠️ Non ti è possibile usare questa funzionalità in quanto non hai nessun Profilo associato all’account. Quando ne viene associato, sarà possibile utilizzare questa funzionalità.`,
            options
          );
          userStates[chatId] = null;
          return;
        }
        await saveUserProfile(chatId, platform, username);
        let options;

        if (platform !== "telegram") {
          options = {
            reply_markup: {
              keyboard: keyboardGuadagna,
              resize_keyboard: true,
            },
          };

          await handleBotMessage(
            userMessages,
            chatId,
            `Il tuo username di ${platform} è stato salvato con successo.`,
            options
          );
        }
      } catch (err) {
        console.error("Errore nel salvataggio del profilo:", err);
      }
    }
  }

  if (
    userMissionStatus[chatId] &&
    userMissionStatus[chatId].awaiting === "problem"
  ) {
    const missionId = userMissionStatus[chatId].missionId;

    try {
      await notifyAdmins(missionId, chatId, msg.text);

      const inlineKeyboard = [
        missionId
          ? { text: "⏩ Avanza", callback_data: "gua-sal-" + missionId }
          : null,
        { text: "↩️ Torna indietro", callback_data: "annulla" },
      ].filter(Boolean); // Filtra gli elementi null

      await handleBotMessage(
        userMessages,
        chatId,
        `✅ Richiesta di assistenza Inviata\n⌛️ Un admin ti risponderà il prima possibile`,
        {
          parse_mode: "HTML",
          reply_markup: {
            inline_keyboard: [inlineKeyboard],
            resize_keyboard: true,
          },
        }
      );
      // Reset the status
      userMissionStatus[chatId] = null;
    } catch (err) {
      console.error("Errore nella segnalazione del problema:", err);
    }
  }

  if (msg.text.startsWith("/admin ")) {
    const messaggio = msg.text.replace("/admin ", "");

    sendMsgAdmin(chatId, messaggio);

    await handleBotMessage(
      userMessages,
      chatId,
      `✅ Messaggio inviato con successo`,
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [
            [{ text: "💼 WALLET" }],
            [{ text: "🗒 GESTIONE DEI POST" }, { text: "⛑ ASSISTENZA" }],
            [{ text: "🗣 RETE DI AMICI" }, { text: "📊 STATISTICHE" }],
            [{ text: "🏆 CLASSIFICHE" }, { text: "🛠 INFO / IMPOSTAZIONI" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      }
    );
  }

  if (msg.text === "⛑ ASSISTENZA") {
    await handleBotMessage(
      userMessages,
      chatId,
      "🚨 Scrivi il problema che hai riscontrato",
      {
        reply_markup: {
          inline_keyboard: [
            [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
          ],
          resize_keyboard: true,
        },
      }
    );

    userMissionStatus[chatId] = {
      awaiting: "problem",
      missionId: null,
    };
  }

  if (
    userMissionStatus[chatId] &&
    userMissionStatus[chatId].awaiting === "description"
  ) {
    const missionId = userMissionStatus[chatId].missionId;

    if (msg.text) {
      try {
        await updateMissionStatus(missionId, userData.id, null, msg.text, null);

        await handleBotMessage(
          userMessages,
          chatId,
          `✔️ Descrizione aggiunta con successo!\n\n<i>Aggiungi e/o aggiorna screenshots/descrizione o clicca sul tasto \n⏩ Avanza</i>`,
          {
            parse_mode: "HTML",
            reply_markup: {
              inline_keyboard: [
                [
                  {
                    text: "🖼 Screenshot",
                    callback_data: "gua-addscreen-" + missionId,
                  },
                  {
                    text: "📜 Descrizione",
                    callback_data: "gua-adddesc-" + missionId,
                  },
                ],
                [{ text: "⏩ Avanza", callback_data: "avanza" }],
                [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
              ],
              resize_keyboard: true,
            },
          }
        );

        // Resetta lo stato
        userMissionStatus[chatId] = null;
      } catch (err) {
        console.error("Errore nell'aggiunta della descrizione:", err);
      }
    }
  }
};

async function notifyAdmins(missionId, userId, problemText) {
  try {
    const admins = await getAdminRoles();
    const messageText = `🚨 Segnalazione problema  ${
      missionId ? `per la missione ID: ${missionId}` : ""
    }\n\nUtente ID: ${userId}\n\nDescrizione del problema:\n${problemText}`;

    for (const admin of admins) {
      if (admin.telegram_id) {
        await bot.sendMessage(admin.telegram_id, messageText, {
          parse_mode: "HTML",
        });
      }
    }
  } catch (err) {
    console.error("Errore durante l'invio della segnalazione agli admin:", err);
  }
}

async function sendMsgAdmin(userId, problemText) {
  try {
    const admins = await getAdminRoles();
    const messageText = `${userId}: ${problemText}`;

    for (const admin of admins) {
      if (admin.telegram_id) {
        await bot.sendMessage(admin.telegram_id, messageText, {
          parse_mode: "HTML",
        });
      }
    }
  } catch (err) {
    console.error("Errore durante l'invio della segnalazione agli admin:", err);
  }
}

const getAdminRoles = () => {
  return new Promise((resolve, reject) => {
    const query = `SELECT telegram_id FROM telegram_users WHERE role = 'admin'`;

    db.query(query, (err, result) => {
      if (err) {
        return reject(err); // Reject the promise if there's an error
      }

      resolve(result); // Resolve the promise with the query result
    });
  });
};

const handlePhoto = async (userMessages, chatId, userData, msg) => {
  if (
    userMissionStatus[chatId] &&
    userMissionStatus[chatId].awaiting === "screenshot"
  ) {
    const missionId = userMissionStatus[chatId].missionId;
    const fileId = msg.photo[msg.photo.length - 1].file_id;

    try {
      await updateMissionStatus(missionId, userData.id, null, null, fileId);

      await handleBotMessage(
        userMessages,
        chatId,
        `✔️ Screenshot aggiunto con successo!\n\n<i>Aggiungi e/o aggiorna screenshots/descrizione o clicca sul tasto \n⏩ Avanza</i>`,
        {
          parse_mode: "HTML",
          reply_markup: {
            inline_keyboard: [
              [
                {
                  text: "🖼 Screenshot",
                  callback_data: "gua-addscreen-" + missionId,
                },
                {
                  text: "📜 Descrizione",
                  callback_data: "gua-adddesc-" + missionId,
                },
              ],
              [{ text: "⏩ Avanza", callback_data: "avanza" }],
              [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
            ],
            resize_keyboard: true,
          },
        }
      );

      // Resetta lo stato
      userMissionStatus[chatId] = null;
    } catch (err) {
      console.error("Errore nell'aggiunta dello screenshot:", err);
    }
  }
};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  if (msg.data === "annulla") {
    userStates[chatId] = null; // Resetta lo stato dell'utente
    userMissionStatus[chatId] = null;
  }

  if (msg.data.startsWith("gua-sal-")) {
    const missionId = msg.data.replace("gua-sal-", "");

    try {
      await updateMissionStatus(missionId, userData.id, "standby");
      await handleBotMessage(
        userMessages,
        chatId,
        "⏸️ Missione messa in standby. Puoi tornare a completarla in un secondo momento."
      );

      await handleAction(userMessages, chatId, userStates[chatId], userData);
    } catch (err) {
      console.error("Errore nel salvataggio della missione in standby:", err);
    }
  }

  if (msg.data.startsWith("gua-skip-")) {
    const missionId = msg.data.replace("gua-skip-", "");

    try {
      await updateMissionStatus(missionId, userData.id, "skip");
      await handleBotMessage(userMessages, chatId, "⏩ Missione saltata!");

      await handleAction(userMessages, chatId, userStates[chatId], userData);
    } catch (err) {
      console.error("Errore nel salvataggio della missione in standby:", err);
    }
  }

  if (msg.data.startsWith("gua-con-")) {
    const missionId = msg.data.replace("gua-con-", "");
    try {
      await updateMissionStatus(missionId, userData.id, "under_review");
      await handleBotMessage(
        userMessages,
        chatId,
        `⌚️ La missione è in fase di verifica!\n\n<i>Se vuoi velocizzere il processo aggiungi screenshot e/o descrizione</i>`,
        {
          parse_mode: "HTML",
          reply_markup: {
            inline_keyboard: [
              [
                {
                  text: "🖼 Aggiungi Screenshot",
                  callback_data: "gua-addscreen-" + missionId,
                },
                {
                  text: "📜 Aggiungi Descrizione",
                  callback_data: "gua-adddesc-" + missionId,
                },
              ],
              [{ text: "⏩ Avanza", callback_data: "avanza" }],
              [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
            ],
            resize_keyboard: true,
          },
        }
      );
    } catch (err) {
      console.error("Errore nel salvataggio della missione completata:", err);
    }
  }

  if (msg.data.startsWith("gua-pubb-")) {
    const missionId = msg.data.replace("gua-pubb-", "");
    try {
      await updateMissionStatus(missionId, userData.id, "complete");

      await handleAction(userMessages, chatId, userStates[chatId], userData);
    } catch (err) {
      console.error("Errore nel salvataggio della missione completata:", err);
    }
  }

  if (msg.data.startsWith("gua-addscreen-")) {
    const missionId = msg.data.replace("gua-addscreen-", "");
    userMissionStatus[chatId] = { awaiting: "screenshot", missionId }; // Memorizza che stiamo aspettando uno screenshot

    await handleBotMessage(
      userMessages,
      chatId,
      "📷 Invia lo screenshot da allegare alla missione:",
      {
        reply_markup: {
          inline_keyboard: [
            [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
          ],
          resize_keyboard: true,
        },
      }
    );
  }

  if (msg.data.startsWith("gua-adddesc-")) {
    const missionId = msg.data.replace("gua-adddesc-", "");
    userMissionStatus[chatId] = { awaiting: "description", missionId }; // Memorizza che stiamo aspettando una descrizione

    await handleBotMessage(
      userMessages,
      chatId,
      "✏️ Invia la descrizione da aggiungere alla missione:",
      {
        reply_markup: {
          inline_keyboard: [
            [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
          ],
          resize_keyboard: true,
        },
      }
    );
  }

  if (msg.data === "avanza") {
    await handleAction(userMessages, chatId, userStates[chatId], userData);
  }

  if (msg.data.startsWith("gua-prob-")) {
    const missionId = await msg.data.replace("gua-prob-", "");

    await handleBotMessage(
      userMessages,
      chatId,
      "🚨 Scrivi il problema che hai riscontrato\n👀 ID Missione: " + missionId,
      {
        reply_markup: {
          inline_keyboard: [
            [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
          ],
          resize_keyboard: true,
        },
      }
    );

    userMissionStatus[chatId] = {
      awaiting: "problem",
      missionId,
    };
  }
};

module.exports = {
  handleGuadagna,
  handleCallbackQuery,
  handlePhoto,
  createMissionMessage,
};
