const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

// Single object to track user's selections (platform, action, points, link)
const userSelections = {};

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const handleMissione = async (userMessages, chatId, userData, msg) => {
  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    userSelections[chatId] = null; // Resetta lo stato dell'utente
  }

  handleUserInput(userMessages, chatId, userData, msg);
  if (msg.text === "➕ INSERISCI MISSIONE ➕") {
    await handleBotMessage(userMessages, chatId, `Seleziona la piattaforma`, {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: [
          [
            { text: "📸 Instagram", callback_data: "addm-instagram" },
            { text: "🟦 Facebook", callback_data: "addm-facebook" },
          ],
          [
            { text: "🎵 TikTok", callback_data: "addm-tiktok" },
            { text: "👤 Telegram", callback_data: "addm-telegram" },
          ],
          [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
        ],
        resize_keyboard: true,
      },
    });
  }
};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  if (msg.data === "annulla") {
    userSelections[chatId] = null; // Resetta lo stato dell'utente
  }

  // Handle platform selection
  if (msg.data.startsWith("addm-")) {
    const platform = msg.data.split("-")[1];
    userSelections[chatId] = { platform }; // Store platform

    let inlineKeyboard;

    // If Telegram, show only "Entra gruppi/canali"
    if (platform === "telegram") {
      inlineKeyboard = [
        [{ text: "Entra gruppi/canali", callback_data: "mission-enter" }],
        [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
      ];
    } else {
      inlineKeyboard = [
        [
          { text: "Segui", callback_data: "mission-follow" },
          { text: "❤️ Like", callback_data: "mission-like" },
        ],
        [
          { text: "💬 Commento", callback_data: "mission-comment" },
          { text: "👀 Visualizzazione Storia", callback_data: "mission-view" },
        ],
        [{ text: "🌍 Condivisione Storia", callback_data: "mission-share" }],
        [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
      ];
    }

    await handleBotMessage(
      userMessages,
      chatId,
      "Indica la Missione in che cosa consisterà.",
      {
        parse_mode: "HTML",
        reply_markup: {
          inline_keyboard: inlineKeyboard,
          resize_keyboard: true,
        },
      }
    );
  }

  // Handle action selection
  if (msg.data.startsWith("mission-")) {
    const action = msg.data.split("-")[1];
    userSelections[chatId].action = action; // Store action

    const { platform } = userSelections[chatId];

    // Query the DB for min/max points for this platform and action
    const query = `
      SELECT points_min, points_max
      FROM platform_action_points_min_max
      WHERE platform = ? AND action_type = ?;
    `;

    db.query(query, [platform, action], async (err, results) => {
      if (err) {
        console.error("Errore nella query dei punti:", err);
        return;
      }

      if (results.length > 0) {
        const { points_min, points_max } = results[0];

        // Ask the user to enter the points within the valid range
        await handleBotMessage(
          userMessages,
          chatId,
          `Per questa piattaforma (${platform}) e azione (${action}), puoi inserire tra ${points_min} e ${points_max} punti.\n\nInserisci il numero di punti:`
        );

        // Store min/max points for validation later
        userSelections[chatId].points_min = points_min;
        userSelections[chatId].points_max = points_max;
      } else {
        await handleBotMessage(
          userMessages,
          chatId,
          `Spiacente, non ci sono punti disponibili per la piattaforma (${platform}) e azione (${action}).`
        );
      }
    });
  }
};

// Handle user's response for points and link
const handleUserInput = async (userMessages, chatId, userData, msg) => {
  const userSelection = userSelections[chatId];

  // Check if user has already entered the points
  if (userSelection && !userSelection.points) {
    const points = parseInt(msg.text, 10);

    // Validate the points
    if (
      isNaN(points) ||
      points < userSelection.points_min ||
      points > userSelection.points_max
    ) {
      await handleBotMessage(
        userMessages,
        chatId,
        `Inserisci un valore di punti valido tra ${userSelection.points_min} e ${userSelection.points_max}.`
      );
    } else {
      userSelection.points = points;

      // Now, ask the user to provide the mission link
      await handleBotMessage(
        userMessages,
        chatId,
        "Perfetto! Ora inviami il link della missione."
      );
    }
  } else if (userSelection && userSelection.points && !userSelection.link) {
    const link = msg.text;

    // Validate the link (basic validation, you can improve this if needed)
    if (!link.startsWith("http")) {
      await handleBotMessage(userMessages, chatId, "Inserisci un link valido.");
    } else {
      userSelection.link = link;

      // Save the mission to the database
      const insertQuery = `
        INSERT INTO missions (platform, action_type, points, link, type, user_id, user_vip, user_admin, status)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?);
      `;

      const status =
        userData.role === "admin"
          ? "active"
          : userData.wallet_missions >= userSelection.points
          ? "pending"
          : "inactive";

      db.query(
        insertQuery,
        [
          userSelection.platform,
          userSelection.action,
          userSelection.points,
          link,
          "normal",
          userData.id,
          userData.role === "user_vip" ? 1 : 0,
          userData.role === "admin" ? 1 : 0,
          status,
        ],
        async (err, result) => {
          if (err) {
            console.error("Errore durante il salvataggio della missione:", err);
            return;
          }

          const keyboard = [
            [{ text: "💶 GUADAGNA 💶" }],
            [{ text: "🛒 SHOP 🛒" }, { text: "👨🏻‍💻 My SocialLife 👨🏻‍💻" }],
            [{ text: "➕ INSERISCI MISSIONE ➕" }],
          ];

          if (userData.role === "admin") {
            keyboard[1].push({ text: "🎓 Admin 🎓" });
          }

          const options = {
            reply_markup: {
              keyboard: keyboard,
              resize_keyboard: true,
            },
          };

          // Success! Send confirmation message
          const missionId = result.insertId;
          await handleBotMessage(
            userMessages,
            chatId,
            `Missione con ID ${missionId} creata con successo, in attesa di approvazione! ${
              userData.wallet_missions < userSelection.points
                ? "Messa in modalità inattiva a causa della mancanza di punti"
                : ""
            }`,
            options
          );

          userSelections[chatId] = null; // Resetta lo stato dell'utente
        }
      );
    }
  }
};

module.exports = {
  handleMissione,
  handleCallbackQuery,
};
