const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });
const moment = require("moment"); // Importa moment.js
const gestionePost = require("./GestionePost.js");

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};
const userStates = {};
const userUpdate = {};

const deleteUser = (uuid) => {
  return new Promise((resolve, reject) => {
    const query = "DELETE FROM `telegram_users` WHERE uuid = ?";
    db.query(query, [uuid], (err, results) => {
      if (err) {
        console.error("Errore nel recupero dell'utente:", err);
        reject(err);
      } else {
        resolve(results[0]);
      }
    });
  });
};

const getUserAndMissionStats = (telegram_id) => {
  return new Promise((resolve, reject) => {
    const query = `
      SELECT 
        tu.created_at, 
        tu.wallet_missions,
        COUNT(ms.id) AS completed_missions_count
      FROM telegram_users tu
      LEFT JOIN mission_status ms ON tu.id = ms.user_id AND ms.status = 'complete'
      WHERE tu.telegram_id = ?
      GROUP BY tu.id
    `;

    db.query(query, [telegram_id], (err, results) => {
      if (err) {
        console.error("Errore nel recupero dell'utente:", err);
        reject(err);
      } else {
        resolve(results[0]);
      }
    });
  });
};

const getLeaderboard = (type) => {
  let query = `
    SELECT tu.telegram_id, SUM(ms.ranking_points) as total_points
    FROM mission_status ms
    LEFT JOIN telegram_users tu ON ms.user_id = tu.id
    WHERE ms.status = 'complete'
  `;

  // Filter based on the leaderboard type
  switch (type) {
    case "🏆 Classifica Giornaliera 🏆":
      query += " AND DATE(ms.timestamp) = CURDATE()";
      break;
    case "🏆 Classifica Settimanale 🏆":
      query += " AND YEARWEEK(ms.timestamp, 1) = YEARWEEK(CURDATE(), 1)";
      break;
    case "🏆 Classifica Mensile 🏆":
      query +=
        " AND MONTH(ms.timestamp) = MONTH(CURDATE()) AND YEAR(ms.timestamp) = YEAR(CURDATE())";
      break;
    default:
      throw new Error("Tipo di classifica non valido");
  }

  query += `
    GROUP BY tu.telegram_id
    ORDER BY total_points DESC
    LIMIT 10;
  `;

  return new Promise((resolve, reject) => {
    db.query(query, (err, results) => {
      if (err) {
        console.error("Errore durante il recupero della classifica:", err);
        reject(err);
      } else {
        resolve(results);
      }
    });
  });
};

const keyboardSocial = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🗒 GESTIONE DEI POST" }, { text: "🛎 NOTIFICHE" }],
      [{ text: "💼 WALLET" }, { text: "⛑ ASSISTENZA" }],
      [{ text: "🗣 RETE DI AMICI" }, { text: "📊 STATISTICHE" }],
      [{ text: "🏆 CLASSIFICHE" }, { text: "🛠 INFO / IMPOSTAZIONI" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const keyboardClassifiche = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [
        { text: "🏆 Classifica Giornaliera 🏆" },
        { text: "🏆 Classifica Settimanale 🏆" },
      ],
      [{ text: "🏆 Classifica Mensile 🏆" }, { text: "🏆 Premi 🏆" }],
      [{ text: "ℹ️ Info ℹ️" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const handleProfileUpdate = async (userMessages, chatId, userData) => {
  const inlineKeyboard = [
    [
      { text: "📸 Instagram", callback_data: "update_instagram" },
      { text: "🎵 TikTok", callback_data: "update_tiktok" },
    ],
    [
      { text: "🟦 Facebook", callback_data: "update_facebook" },
      { text: "👉🏻 Genere", callback_data: "update_gender" },
    ],
    [
      { text: "👉🏻 Età", callback_data: "update_age" },
      { text: "👉🏻 Device", callback_data: "update_device" },
    ],
    [{ text: "🔄 Aggiorna", callback_data: "back" }],
    [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
  ];

  const options = {
    parse_mode: "HTML",
    reply_markup: {
      inline_keyboard: inlineKeyboard,
      resize_keyboard: true,
    },
  };

  await handleBotMessage(
    userMessages,
    chatId,
    `👤 <b>Profilo</b>

🤖 Telegram ID: ${userData.telegram_id}
📸 Instagram: ${userData.instagram ?? ""}
🟦 Facebook: ${userData.facebook ?? ""}
🎵 TikTok: ${userData.tiktok ?? ""}

👉🏻 Genere: ${userData.gender === "male" ? "Uomo" : "Donna"}
👉🏻 Età: ${userData.age_range}
👉🏻 Device: ${userData.device}`,
    options
  );
};

const getInvitedCount = async (userId) => {
  try {
    const [rows] = await db
      .promise()
      .query(
        "SELECT COUNT(*) AS count FROM telegram_users WHERE invitedById = ?",
        [userId]
      );

    return rows[0].count;
  } catch (error) {
    console.error("Errore durante il conteggio degli invitati:", error);
    throw error;
  }
};

const handleMySocial = async (userMessages, chatId, userData, msg) => {
  gestionePost.handlePost(userMessages, chatId, userData, msg);

  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    userStates[chatId] = null; // Resetta lo stato dell'utente
    userUpdate[chatId] = null;
  }

  if (msg.text === "📊 STATISTICHE") {
    const statisticheInfo = await getUserAndMissionStats(chatId);
    const formattedDate = moment(statisticheInfo.created_at).format(
      "DD/MM/YYYY HH:mm"
    );

    const statistiche = `📊 Statistiche 📊

In questa schermata potrai vedere alcune statistiche relative al tuo account.

<b>Missioni Completate:</b> ${statisticheInfo.completed_missions_count}
<b>Punti Accumulati:</b> ${statisticheInfo.wallet_missions}
<b>Data di Registrazione:</b> ${formattedDate}`;
    await handleBotMessage(userMessages, chatId, statistiche, keyboardSocial);
  }

  if (msg.text === "👨🏻‍💻 My SocialLife 👨🏻‍💻") {
    await handleBotMessage(
      userMessages,
      chatId,
      "Benvenuto nella tua area personale. Qui puoi gestire le missioni, monitorare i tuoi guadagni, aggiornare le tue informazioni e molto altro.\n\n⬇️ Seleziona una delle opzioni sottostanti per navigare:",
      keyboardSocial
    );
  }

  if (msg.text === "🏆 Premi 🏆") {
    await handleBotMessage(
      userMessages,
      chatId,
      `💶 Premi per i vincitori delle Classifiche:

🏆 Classifica Giornaliera
🥇 Posto: 1000 punti
🥈 Posto: 500 punti
🥉 Posto: 250 punti

🏆 Classifica Settimanale
🥇 Posto: 5.000 punti
🥈 Posto: 2.500 punti
🥉 Posto: 1.250 punti

🏆 Classifica Mensile
🥇 Posto: 10.000 punti
🥈 Posto: 5.000 punti
🥉 Posto: 2.500 punti
`,
      keyboardClassifiche
    );
  }

  if (msg.text === "🏆 CLASSIFICHE") {
    await handleBotMessage(
      userMessages,
      chatId,
      `🏆 <b>Classifiche</b> 🏆

Scala le classifiche di SocialLife ed ottieni premi aggiuntivi. Per ogni missione completata, guadagnerai punti che ti permetteranno di avanzare nelle classifiche.

Per ulteriori informazioni, visita la sezione “Info”.`,
      keyboardClassifiche
    );
  }

  if (msg.text.startsWith("🏆 Classifica ")) {
    const classifica = msg.text;

    try {
      const leaderboard = await getLeaderboard(classifica);

      let message = `<b>${classifica}</b>\n\n`;
      let userRank = null;
      let userTotalPoints = 0;

      if (leaderboard.length === 0) {
        message += "Nessun risultato disponibile.\n";
      } else {
        leaderboard.forEach((entry, index) => {
          let medal = "";
          if (index === 0) medal = "🥇";
          if (index === 1) medal = "🥈";
          if (index === 2) medal = "🥉";

          const isCurrentUser = entry.telegram_id == chatId ? " (TU)" : "";
          message += `${index + 1}. ID ${entry.telegram_id} Punti ${
            entry.total_points
          } ${medal}${isCurrentUser}\n`;

          // Track the current user's position and points
          if (entry.telegram_id == chatId) {
            userRank = index + 1;
            userTotalPoints = entry.total_points;
          }
        });

        // If user is not in the top 10, search for their total points and position
        if (!userRank) {
          const userEntry = leaderboard.find(
            (entry) => entry.telegram_id == chatId
          );
          if (userEntry) {
            userRank = leaderboard.indexOf(userEntry) + 1;
            userTotalPoints = userEntry.total_points;
          }
        }
      }

      // Add the personalized message for the user at the end
      if (userRank !== null) {
        message += `\nCiao, <b>${userData.first_name}</b>! Hai accumulato un totale di ${userTotalPoints} punti e ti trovi in ${userRank}ª posizione. Non ti fermare! 🚀`;
      } else {
        message += `\nCiao, <b>${userData.first_name}</b>! Non hai ancora accumulato punti. Inizia subito per scalare le classifiche! 🚀`;
      }

      await handleBotMessage(
        userMessages,
        chatId,
        message,
        keyboardClassifiche
      );
    } catch (err) {
      console.error("Errore durante la gestione della classifica:", err);
      await handleBotMessage(
        userMessages,
        chatId,
        "Errore durante la generazione della classifica."
      );
    }
  }

  if (msg.text === "ℹ️ Info ℹ️") {
    await handleBotMessage(
      userMessages,
      chatId,
      `🏆 <b>Classifiche</b> 🏆

Scala le classifiche di SocialLife e ottieni premi esclusivi.

<b>Punti per le Missioni:</b>

 • 👀 Guarda Storie / Reel: 1 punto
 • 👍 Metti Mi Piace ai post / Reel: 2 punti
 • 💬 Commenta sui post / Reel: 3 punti
 • ➕ Segui nuovi utenti: 5 punti
 • 👥 Invita amici: 20 punti

Inizia subito! Completa tutte le missioni e vinci premi incredibili!`,
      keyboardClassifiche
    );
  }

  if (msg.text === "🗣 RETE DI AMICI") {
    const invitedCount = await getInvitedCount(userData.id);
    await handleBotMessage(
      userMessages,
      chatId,
      "👥 Rete di Amici 👥\n\n" +
        "È il momento di far crescere la tua rete e i tuoi premi! 🚀\n" +
        "Invita i tuoi amici e ricevi il 20% dei punti che guadagnano! \n\n" +
        "Condividi il seguente link con i tuoi amici ⬇️\n" +
        "Link: https://t.me/Socialife_bot?start=ref_" +
        chatId +
        "\n\n" +
        "Offerta Premium Incredibile! 🌟\n" +
        "Vuoi ancora di più? Con i nostri pacchetti “Premium”, puoi guadagnare fino al 35% dei punti guadagnati da ogni amico invitato! Sì, hai letto bene! 🔥\n\n" +
        "➡️ Visita la sezione “Shop” per maggiori dettagli.\n\n" +
        "Stato e Rete Attuale\n" +
        "👨🏻‍💻 Status: " +
        userData.role +
        "\n" +
        `👥 Rete Attuale: ${invitedCount}\n` +
        `💶 Guadagno: ${userData.role === "user_vip" ? "35%" : "20%"}\n\n` +
        "Tieni traccia dei tuoi punti nella sezione “Wallet”.",
      keyboardSocial
    );
  }

  if (msg.text === "🛠 INFO / IMPOSTAZIONI") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "⚙️ Impostazioni" }, { text: "📜 Regolamento" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `<b>Non perdere questa occasione! SocialLife ti premia per completare le missioni!</b> 🚀🔥

Tipi di Missioni:

 👍 <b>Mi piace</b> sui post
 ❤️ <b>Mi piace</b> sulle storie
 💬 <b>Commenti</b> sui post
 👀 <b>Visualizzazioni</b> delle storie
 🎯 <b>e molto altro</b>…

<b>Completa le missioni, accumula punti e convertili subito in denaro o gift card! 💰🎁 Non aspettare, inizia ora e fai il pieno di premi!</b> 🏆🚀`,
      options
    );
  }

  if (msg.text === "📜 Regolamento") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "⚙️ Impostazioni" }, { text: "📜 Regolamento" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `<b>Regolamento</b>

Per garantire un’esperienza ottimale, si prega di rispettare le seguenti regole:

✅ <b>Permessi:</b>

 1. Account reali e attivi

⛔️ <b>Proibizioni:</b>

 1. Uso di account falsi o inattivi
 2. Uso di account di citazioni, meme o simili
 3. Commenti con una singola emoticon o meno di 2 parole

⚠️ <b>Avviso Importante:</b>

Le attività che generano credito non devono essere rimosse, nemmeno dopo il pagamento. Il mancato rispetto di queste regole comporterà il BAN dell’account senza preavviso e senza possibilità di reiscrizione.`,
      options
    );
  }

  if (msg.text === "⚙️ Impostazioni") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "👤 Profilo" }, { text: "❌ Elimina Account" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `⚙️ <b>Impostazioni</b>

In questa sezione potrai richiedere supporto oppure eliminare il tuo account.`,
      options
    );
  }

  if (msg.text === "👤 Profilo") {
    await handleProfileUpdate(userMessages, chatId, userData);
  }

  if (userStates[chatId] === "delete") {
    userStates[chatId] = "delete-" + msg.text;

    const options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: [
          [
            { text: "✅ Si", callback_data: "delete-si" },
            { text: "❌ No", callback_data: "acc-annulla" },
          ],
        ],
      },
    };

    await handleBotMessage(
      userMessages,
      chatId,
      `Confermi di voler eliminare il tuo account?`,
      options
    );
  }

  if (msg.text === "❌ Elimina Account") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: [[{ text: "❌ Annulla", callback_data: "annulla" }]],
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Inserisci la tua <b>Personal Key</b> ricevuta al momento della registrazione.

<b>NB. Una volta inviata la tua Personal Key, l’account verrà cancellato senza poterlo più recuperare.</b>`,
      options
    );

    userStates[chatId] = "delete";
  }

  if (msg.text === "🆘 Supporto") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "🆘 Supporto" }, { text: "❌ Elimina Account" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `<b>Supporto</b>

🆘 <b>Hai bisogno di assistenza?</b>

Per supporto su malfunzionamenti del bot o altre informazioni, contatta @XXXXXXXXXXXXXX con i tuoi dati di registrazione.

⚠️ <b>Nota</b>: La tua Personal Key non può essere recuperata in nessun caso, nemmeno contattando lo Staff. Rispondiamo entro 12/24 ore, a seconda del carico di lavoro.`,
      options
    );
  }

  if (userUpdate[chatId]) {
    const field = userUpdate[chatId];
    const newValue = msg.text.trim();

    // Aggiornamento del database
    let updateField;
    switch (field) {
      case "instagram":
        updateField = "instagram";
        break;
      case "tiktok":
        updateField = "tiktok";
        break;
      case "facebook":
        updateField = "facebook";
        break;
      case "gender":
        updateField = "gender";
        break;
      case "age":
        updateField = "age_range";
        break;
      case "device":
        updateField = "device";
        break;
    }

    // Esegui l'update nel database
    const updateQuery = `UPDATE telegram_users SET ${updateField} = ? WHERE telegram_id = ?`;
    await db.promise().query(updateQuery, [newValue, userData.telegram_id]);

    // Notifica l'utente
    await handleBotMessage(
      userMessages,
      chatId,
      `✅ Profilo aggiornato: ${updateField} è stato modificato.\nPer visualizzare le modifiche utilizza il tasto 🔄 Aggiorna `
    );

    await handleProfileUpdate(userMessages, chatId, userData);

    // Resetta lo stato dell'utente
    delete userUpdate[chatId];
  }
};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  gestionePost.handleCallbackQuery(userMessages, chatId, userData, msg);

  if (msg.data === "acc-annulla") {
    await handleBotMessage(
      userMessages,
      chatId,
      `❌ <b>Eliminazione Account Annullata!</b>`,
      {
        parse_mode: "HTML",
      }
    );
    userStates[chatId] = null;
  }

  if (msg.data === "delete-si") {
    const uuid = userStates[chatId].replace("delete-", "");
    await deleteUser(uuid);

    await handleBotMessage(
      userMessages,
      chatId,
      `✅ <b>Il tuo account è stato eliminato!</b>`,
      {
        parse_mode: "HTML",
      }
    );
    userStates[chatId] = null;
  }
  let text, options;

  if (msg.data.startsWith("age_")) {
    const age = msg.data.split("_");
    const ageUpdate = age[1] + "-" + age[2];
    const updateQuery = `UPDATE telegram_users SET age_range = ? WHERE telegram_id = ?`;
    await db.promise().query(updateQuery, [ageUpdate, userData.telegram_id]);

    await handleBotMessage(
      userMessages,
      chatId,
      `✅ Profilo aggiornato: Età aggiornata.\nPer visualizzare le modifiche utilizza il tasto 🔄 Aggiorna `
    );

    await handleProfileUpdate(userMessages, chatId, userData);
    delete userUpdate[chatId];
  }

  if (msg.data.startsWith("gender_")) {
    const gender = msg.data.split("_");
    const updateQuery = `UPDATE telegram_users SET gender = ? WHERE telegram_id = ?`;
    await db.promise().query(updateQuery, [gender[1], userData.telegram_id]);

    await handleBotMessage(
      userMessages,
      chatId,
      `✅ Profilo aggiornato: Genere aggiornato.\nPer visualizzare le modifiche utilizza il tasto 🔄 Aggiorna `
    );

    await handleProfileUpdate(userMessages, chatId, userData);
    delete userUpdate[chatId];
  }

  if (msg.data.startsWith("device_")) {
    const device = msg.data.split("_");
    const updateQuery = `UPDATE telegram_users SET device = ? WHERE telegram_id = ?`;
    await db.promise().query(updateQuery, [device[1], userData.telegram_id]);

    await handleBotMessage(
      userMessages,
      chatId,
      `✅ Profilo aggiornato: Dispositivo aggiornato.\nPer visualizzare le modifiche utilizza il tasto 🔄 Aggiorna `
    );

    await handleProfileUpdate(userMessages, chatId, userData);
    delete userUpdate[chatId];
  }
  const optionsBack = {
    reply_markup: {
      keyboard: [[{ text: "↩️ Torna indietro" }]],
      resize_keyboard: true,
    },
  };
  switch (msg.data) {
    case "update_instagram":
      userUpdate[chatId] = "instagram";
      text = "📸 Invia ora il nuovo profilo Instagram:";
      await handleBotMessage(userMessages, chatId, text, optionsBack);
      break;
    case "update_tiktok":
      userUpdate[chatId] = "tiktok";
      text = "🎵 Invia ora il nuovo profilo TikTok:";
      await handleBotMessage(userMessages, chatId, text, optionsBack);
      break;
    case "update_facebook":
      userUpdate[chatId] = "facebook";
      text = "🟦 Invia ora il nuovo profilo Facebook:";
      await handleBotMessage(userMessages, chatId, text, optionsBack);
      break;
    case "update_gender":
      userUpdate[chatId] = "gender";
      text = "👉🏻 Scegli il tuo genere:";
      options = {
        reply_markup: {
          inline_keyboard: [
            [{ text: "Uomo", callback_data: "gender_male" }],
            [{ text: "Donna", callback_data: "gender_female" }],
          ],
        },
      };
      await handleBotMessage(userMessages, chatId, text, options);
      break;
    case "update_age":
      userUpdate[chatId] = "age";
      text = "👉🏻 Scegli la tua età:";
      options = {
        reply_markup: {
          inline_keyboard: [
            [{ text: "0-18", callback_data: "age_0_18" }],
            [{ text: "18-25", callback_data: "age_18_25" }],
            [{ text: "25-30", callback_data: "age_25_30" }],
            [{ text: "30-40", callback_data: "age_30_40" }],
            [{ text: "40+", callback_data: "age_40_plus" }],
          ],
        },
      };
      await handleBotMessage(userMessages, chatId, text, options);
      break;
    case "update_device":
      userUpdate[chatId] = "device";
      text = "👉🏻 Scegli il tuo dispositivo:";
      options = {
        reply_markup: {
          inline_keyboard: [
            [{ text: "Apple", callback_data: "device_apple" }],
            [{ text: "Android", callback_data: "device_android" }],
          ],
        },
      };
      await handleBotMessage(userMessages, chatId, text, options);
      break;
    case "back":
      await handleProfileUpdate(userMessages, chatId, userData);
      return;
    default:
      return;
  }
};

module.exports = {
  handleMySocial,
  handleCallbackQuery,
};
