require("dotenv").config();
const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const { v4: uuidv4 } = require("uuid");
const guadagna = require("./Guadagna.js");
const wallet = require("./Wallet.js");
const mysocial = require("./Mysocial.js");
const shop = require("./Shop.js");
const admin = require("./Admin.js");
const inserisciMissione = require("./InserisciMissione.js");

const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: true });

const userMessages = {};

bot.on("polling_error", (error) => {
  console.error("[polling_error]", error);
});

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const getUser = (userId) => {
  return new Promise((resolve, reject) => {
    const query = "SELECT * FROM telegram_users WHERE telegram_id = ?";
    db.query(query, [userId], (err, results) => {
      if (err) {
        console.error("Errore nel recupero dell'utente:", err);
        reject(err);
      } else {
        resolve(results[0]);
      }
    });
  });
};

const generateAndSaveUUID = (userId) => {
  const newUUID = uuidv4();
  return new Promise((resolve, reject) => {
    const updateQuery =
      "UPDATE telegram_users SET uuid = ? WHERE telegram_id = ?";
    db.execute(updateQuery, [newUUID, userId], (err, results) => {
      if (err) {
        console.error("Errore nel salvare l'UUID:", err);
        reject(err);
      } else {
        resolve(newUUID);
      }
    });
  });
};

const saveUser = (user, refId) => {
  return new Promise((resolve, reject) => {
    const { id, first_name, last_name, username } = user;
    const firstName = first_name || null;
    const lastName = last_name || null;
    const userName = username || null;

    const checkQuery = "SELECT id FROM telegram_users WHERE telegram_id = ?";
    db.query(checkQuery, [id], async (err, results) => {
      if (err) {
        console.error("Errore nella verifica dell'utente:", err);
        reject(err);
      } else if (results.length === 0) {
        const ref = refId ? await getUser(refId) : null;
        const insertQuery = `
          INSERT INTO telegram_users (telegram_id, first_name, last_name, username, invitedById )
          VALUES (?, ?, ?, ?, ?)
        `;
        db.execute(
          insertQuery,
          [id, firstName, lastName, userName, ref?.id ?? null],
          (err, results) => {
            if (err) {
              console.error("Errore nel salvare utente:", err);
              reject(err);
            } else {
              console.log("Utente creato:", results);
              resolve(results);
            }
          }
        );
      } else {
        const updateQuery = `
          UPDATE telegram_users
          SET first_name = ?, last_name = ?, username = ?
          WHERE telegram_id = ?
        `;
        db.execute(
          updateQuery,
          [firstName, lastName, userName, id],
          (err, results) => {
            if (err) {
              console.error("Errore nell'aggiornare utente:", err);
              reject(err);
            } else {
              console.log("Utente aggiornato:", results);
              resolve(results);
            }
          }
        );
      }
    });
  });
};

const updateUserInfo = (chatId, userId, field, value) => {
  const updateQuery = `UPDATE telegram_users SET ${field} = ? WHERE telegram_id = ?`;
  db.execute(updateQuery, [value, userId], (err, results) => {
    if (err) {
      console.error("Errore nell'aggiornare le informazioni:", err);
      handleBotMessage(
        chatId,
        "Si è verificato un errore durante l'aggiornamento delle informazioni."
      );
    } else {
      handleBotMessage(chatId, `Informazione aggiornata con successo.`);
      checkAndCompleteRegistration(chatId, { id: userId });
    }
  });
};

const getUpdateOptions = (userData) => {
  const keyboard = [
    [{ text: "Aggiorna Genere" }, { text: "Aggiorna Età" }],
    [{ text: "Aggiorna Dispositivo" }],
    [{ text: "↩️ Torna indietro" }],
  ];

  if (userData.gender)
    keyboard[0] = keyboard[0].filter((btn) => btn.text !== "Aggiorna Genere");
  if (userData.age_range)
    keyboard[0] = keyboard[0].filter((btn) => btn.text !== "Aggiorna Età");
  if (userData.device)
    keyboard[1] = keyboard[1].filter(
      (btn) => btn.text !== "Aggiorna Dispositivo"
    );

  return {
    reply_markup: {
      keyboard:
        keyboard.length > 0 ? keyboard : [[{ text: "↩️ Torna indietro" }]],
      resize_keyboard: true,
    },
  };
};

const handleUpdateSelection = (chatId, text) => {
  switch (text) {
    case "Aggiorna Genere":
      handleBotMessage(chatId, "Scegli il tuo genere:", {
        reply_markup: {
          keyboard: [
            [{ text: "🧔‍♂️ Uomo" }, { text: "🧔‍♀️ Donna" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      });
      break;
    case "Aggiorna Età":
      handleBotMessage(chatId, "Scegli la tua fascia d'età:", {
        reply_markup: {
          keyboard: [
            [{ text: "0-18" }, { text: "18-25" }],
            [{ text: "25-30" }, { text: "30-40" }],
            [{ text: "40+" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      });
      break;
    case "Aggiorna Dispositivo":
      handleBotMessage(chatId, "Scegli il tuo dispositivo:", {
        reply_markup: {
          keyboard: [
            [{ text: "Android" }, { text: "Apple" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      });
      break;
    default:
      // handleBotMessage(chatId, "Opzione non riconosciuta.");
      break;
  }
};

const checkAndCompleteRegistration = (chatId, user) => {
  const { id } = user;

  getUser(id)
    .then((userData) => {
      if (!userData.gender || !userData.age_range || !userData.device) {
        handleBotMessage(
          chatId,
          "Completa la tua registrazione per accedere a tutte le funzionalità:",
          getUpdateOptions(userData)
        );
      } else {
        generateAndSaveUUID(id)
          .then((newUUID) => {
            handleBotMessage(
              chatId,
              `<b>Registrazione completata con successo!</b> 🥳

Ecco la tua Super Key Personale: <b>${newUUID}</b>

Questa chiave magica ti servirà per aggiornare le tue info, cancellare il tuo account e richiedere i tuoi pagamenti. Attenzione! La tua Super Key non può essere recuperata, neanche dai nostri maghi del supporto, quindi trattala come un tesoro prezioso! 💎\n\nDigita /start per iniziare la tua avventura`,
              { parse_mode: "HTML" }
            );
          })
          .catch((error) => {
            console.error("Errore nella generazione dell'UUID:", error);
            handleBotMessage(
              chatId,
              "Si è verificato un errore durante la registrazione."
            );
          });
      }
    })
    .catch((error) => {
      console.error("Errore nella verifica dell'utente:", error);
      handleBotMessage(
        chatId,
        "Si è verificato un errore durante la verifica dell'utente."
      );
    });
};

const deletePreviousMessages = (chatId) => {
  if (!userMessages[chatId]) {
    userMessages[chatId] = [];
  }

  const messageIds = userMessages[chatId];
  messageIds.forEach((messageId) => {
    setTimeout(() => {
      bot.deleteMessage(chatId, messageId).catch(console.error);
    }, 300);
  });

  userMessages[chatId] = [];
};

const handleBotMessage = async (chatId, text, options) => {
  const sentMessage = await bot.sendMessage(chatId, text, options);

  if (!userMessages[chatId]) {
    userMessages[chatId] = [];
  }

  userMessages[chatId].push(sentMessage.message_id);
};
bot.on("message", async (msg) => {
  const chatId = msg.chat.id;
  const userId = msg.from.id;

  try {
    deletePreviousMessages(chatId);

    const userData = await getUser(userId);

    if (
      msg.text === "/start" ||
      msg.text === "↩️ Torna indietro" ||
      msg.text?.startsWith("/start")
    ) {
      const startCommand = msg.text.startsWith("/start")
        ? msg.text.split(" ")
        : [];
      const queryParams = startCommand[1]
        ? startCommand[1].replace("ref_", "")
        : null;
      const refId = queryParams || null;

      await saveUser(msg.from, refId);

      const userData = await getUser(userId);
      if (userData.uuid) {
        const keyboard = [
          [{ text: "💶 GUADAGNA 💶" }],
          [{ text: "🛒 SHOP 🛒" }, { text: "👨🏻‍💻 My SocialLife 👨🏻‍💻" }],
          [{ text: "➕ INSERISCI MISSIONE ➕" }],
        ];

        if (userData.role === "admin") {
          keyboard[1].push({ text: "🎓 Admin 🎓" });
        }

        const options = {
          reply_markup: {
            keyboard: keyboard,
            resize_keyboard: true,
          },
        };
        await handleBotMessage(
          chatId,
          "👋 Benvenuto su SocialLife!\nScegli uno dei menù per spostarti all’interno del bot. ⬇️",
          options
        );
      } else {
        checkAndCompleteRegistration(chatId, { id: userId });
      }
    } else if (
      [
        "🧔‍♂️ Uomo",
        "🧔‍♀️ Donna",
        "0-18",
        "18-25",
        "25-30",
        "30-40",
        "40+",
        "Android",
        "Apple",
      ].includes(msg.text)
    ) {
      let field;

      if (["🧔‍♂️ Uomo", "🧔‍♀️ Donna"].includes(msg.text)) {
        field = "gender";
        msg.text = msg.text === "🧔‍♂️ Uomo" ? "male" : "female";
      } else if (
        ["0-18", "18-25", "25-30", "30-40", "40+"].includes(msg.text)
      ) {
        field = "age_range";
      } else if (["Android", "Apple"].includes(msg.text)) {
        field = "device";
      }

      if (field) {
        await updateUserInfo(chatId, userId, field, msg.text);
      }
    } else {
      handleUpdateSelection(chatId, msg.text);
    }

    bot.deleteMessage(chatId, msg.message_id).catch(console.error);
    guadagna.handleGuadagna(userMessages, chatId, userData, msg);
    shop.handleShop(userMessages, chatId, userData, msg);
    mysocial.handleMySocial(userMessages, chatId, userData, msg);
    wallet.handleWallet(userMessages, chatId, userData, msg);
    inserisciMissione.handleMissione(userMessages, chatId, userData, msg);

    if (userData?.role === "admin") {
      admin.handleAdmin(userMessages, chatId, userData, msg);
    }
  } catch (error) {
    console.error("Errore nella gestione del comando:", error);
    handleBotMessage(chatId, "Si è verificato un errore.");
  }
});

const handleCallbackQuery = async (callbackQuery) => {
  const chatId = callbackQuery.message.chat.id;

  try {
    const userData = await getUser(chatId);

    deletePreviousMessages(chatId);

    shop.handleCallbackQuery(userMessages, chatId, userData, callbackQuery);
    guadagna.handleCallbackQuery(userMessages, chatId, userData, callbackQuery);
    mysocial.handleCallbackQuery(userMessages, chatId, userData, callbackQuery);
    admin.handleCallbackQuery(userMessages, chatId, userData, callbackQuery);
    inserisciMissione.handleCallbackQuery(
      userMessages,
      chatId,
      userData,
      callbackQuery
    );

    if (
      callbackQuery.data === "annulla" ||
      callbackQuery.data === "acc-annulla"
    ) {
      const keyboard = [
        [{ text: "💶 GUADAGNA 💶" }],
        [{ text: "🛒 SHOP 🛒" }, { text: "👨🏻‍💻 My SocialLife 👨🏻‍💻" }],
        [{ text: "➕ INSERISCI MISSIONE ➕" }],
      ];

      if (userData.role === "admin") {
        keyboard[1].push({ text: "🎓 Admin 🎓" });
      }

      await handleBotMessage(
        chatId,
        "👋 Benvenuto su SocialLife!\nScegli uno dei menù per spostarti all’interno del bot. ⬇️",
        {
          reply_markup: {
            keyboard: keyboard,
            resize_keyboard: true,
          },
        }
      );
    }
  } catch (error) {
    console.error("Errore nella gestione del comando:", error);
    handleBotMessage(chatId, "Si è verificato un errore.");
  }
};

bot.on("callback_query", handleCallbackQuery);

bot.on("photo", async (photoMsg) => {
  const chatId = photoMsg.chat.id;

  try {
    const userData = await getUser(chatId);

    deletePreviousMessages(chatId);

    guadagna.handlePhoto(userMessages, chatId, userData, photoMsg);
  } catch (error) {
    console.error("Errore nella gestione del comando:", error);
    handleBotMessage(chatId, "Si è verificato un errore.");
  }
});
