require("dotenv").config();
const mysql = require("mysql2/promise");
const cron = require("node-cron");
const TelegramBot = require("node-telegram-bot-api");

// Create database connection
const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

// Create Telegram bot instance
const bot = new TelegramBot(process.env.TELEGRAM_BOT_TOKEN, { polling: false });

// Leaderboard types and corresponding rewards
const leaderboardRewards = {
  daily: [1000, 500, 250],
  weekly: [5000, 2500, 1250],
  monthly: [10000, 5000, 2500],
};

// Congratulation messages
const getCongratulationMessage = (position, type, points) => {
  const typeText = {
    daily: "giornaliera 🏆",
    weekly: "settimanale 🏆",
    monthly: "mensile 🏆",
  }[type];

  const positions = ["primo 🥇", "secondo 🥈", "terzo 🥉"];

  return `🎉 Complimenti! Sei arrivato ${positions[position]} nella classifica ${typeText} e hai ricevuto ${points} punti! 💪`;
};

// Get leaderboard query based on type
const getLeaderboardQuery = (type) => {
  let query = `
    SELECT tu.id, tu.telegram_id, SUM(ms.ranking_points) AS total_points
    FROM mission_status ms
    LEFT JOIN telegram_users tu ON ms.user_id = tu.id
    WHERE ms.status = 'complete'
  `;

  switch (type) {
    case "daily":
      query += " AND DATE(ms.timestamp) = CURDATE()";
      break;
    case "weekly":
      query += " AND YEARWEEK(ms.timestamp, 1) = YEARWEEK(CURDATE(), 1)";
      break;
    case "monthly":
      query +=
        " AND MONTH(ms.timestamp) = MONTH(CURDATE()) AND YEAR(ms.timestamp) = YEAR(CURDATE())";
      break;
    default:
      throw new Error("Invalid leaderboard type");
  }

  query += `
    GROUP BY tu.id
    ORDER BY total_points DESC
    LIMIT 3;
  `;

  return query;
};

// Update wallet_missions
const updateWalletMissions = async (userId, points) => {
  try {
    const [rows] = await db.execute(
      `UPDATE telegram_users SET wallet_missions = wallet_missions + ? WHERE id = ?`,
      [points, userId]
    );
    return rows;
  } catch (err) {
    console.error("Error updating wallet_missions:", err);
    throw err;
  }
};

// Send congratulation message
const sendCongratulationMessage = async (
  telegramId,
  position,
  type,
  points
) => {
  const message = getCongratulationMessage(position, type, points);
  try {
    await bot.sendMessage(telegramId, message);
    console.log(`Sent message to user ${telegramId}: ${message}`);
  } catch (err) {
    console.error(`Error sending message to user ${telegramId}:`, err);
  }
};

// Process leaderboard and reward users
const processLeaderboard = async (type) => {
  try {
    const connection = await db;
    const query = getLeaderboardQuery(type);
    const [leaderboard] = await connection.query(query);

    const rewards = leaderboardRewards[type];

    // Update points for the top 3 users and send congratulatory messages
    for (let i = 0; i < leaderboard.length; i++) {
      const user = leaderboard[i];
      const points = rewards[i];
      console.log(`Rewarding ${points} points to user ${user.telegram_id}`);
      await updateWalletMissions(user.id, points);
      await sendCongratulationMessage(user.telegram_id, i, type, points);
    }

    console.log(`${type} leaderboard processed successfully.`);
  } catch (err) {
    console.error(`Error processing ${type} leaderboard:`, err);
  }
};

// Schedule cron jobs
cron.schedule("0 0 * * *", async () => {
  // Daily leaderboard at midnight (00:00 every day)
  await processLeaderboard("daily");
  console.log("🏆 Daily leaderboard processed.");
});

cron.schedule("0 0 * * 1", async () => {
  // Weekly leaderboard at midnight on Monday (00:00 every Monday)
  await processLeaderboard("weekly");
  console.log("🏆 Weekly leaderboard processed.");
});

cron.schedule("0 0 1 * *", async () => {
  // Monthly leaderboard at midnight on the 1st of every month (00:00 on the 1st of each month)
  await processLeaderboard("monthly");
  console.log("🏆 Monthly leaderboard processed.");
});

// Start the server or other logic if needed
