require("dotenv").config();
const mysql = require("mysql2/promise"); // Usa mysql2/promise per supportare async/await

// Crea una connessione MySQL utilizzando i dati dal file .env
const createConnection = async () => {
  return mysql.createConnection({
    host: process.env.DB_HOST,
    user: process.env.DB_USER,
    password: process.env.DB_PASSWORD,
    database: process.env.DB_NAME,
  });
};

// Creazione della tabella telegram_users
const createTelegramUsersTable = `
CREATE TABLE IF NOT EXISTS telegram_users (
    id INT AUTO_INCREMENT PRIMARY KEY,
    uuid VARCHAR(36) DEFAULT NULL,
    wallet DECIMAL(10, 3) DEFAULT 0,
    wallet_missions DECIMAL(10, 2) DEFAULT 0,
    wallet_buy DECIMAL(10, 2) DEFAULT 0,
    wallet_friends DECIMAL(10, 2) DEFAULT 0,
    telegram_id BIGINT NOT NULL UNIQUE,
    first_name VARCHAR(255),
    last_name VARCHAR(255),
    username VARCHAR(255),
    role ENUM('user', 'user_vip', 'admin') DEFAULT 'user',
    facebook TEXT DEFAULT NULL,
    instagram TEXT DEFAULT NULL,
    tiktok TEXT DEFAULT NULL,
    gender ENUM('male', 'female') DEFAULT NULL,
    age_range ENUM('0-18', '18-25', '25-30', '30-40', '40+') DEFAULT NULL,
    device ENUM('Android', 'Apple') DEFAULT NULL,
    invitedById INT DEFAULT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (invitedById) REFERENCES telegram_users(id) ON DELETE SET NULL
);
`;

// Creazione della tabella missions
const createMissionsTable = `
CREATE TABLE IF NOT EXISTS missions (
    id INT AUTO_INCREMENT PRIMARY KEY,
    link TEXT NOT NULL,
    instructions TEXT NULL DEFAULT NULL,
    start TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    end TIMESTAMP NULL,
    type ENUM('normal', 'time', 'close_number', 'advertising') NOT NULL,
    platform ENUM('instagram', 'facebook', 'telegram', 'tiktok') NULL DEFAULT NULL,
    points INT NOT NULL,
    action_type ENUM('enter', 'like', 'follow', 'comment', 'share', 'view') NULL DEFAULT NULL,
    status ENUM('pending','active', 'inactive') DEFAULT 'pending',
    approved BOOLEAN DEFAULT FALSE,
    user_vip BOOLEAN DEFAULT FALSE,
    user_admin BOOLEAN DEFAULT FALSE,
    user_id INT NOT NULL,
    max_user INT NULL DEFAULT NULL,
    count_user INT NULL DEFAULT NULL,
    FOREIGN KEY (user_id) REFERENCES telegram_users(id)
);
`;

// Creazione della tabella mission_status
const createMissionStatusTable = `
CREATE TABLE IF NOT EXISTS mission_status (
    id INT AUTO_INCREMENT PRIMARY KEY,
    mission_id INT NOT NULL,
    user_id INT NOT NULL,
    status ENUM('skip','standby', 'under_review', 'rejected','complete') DEFAULT 'under_review',
    ranking_points INT NOT NULL,
    text_added TEXT NULL DEFAULT NULL,
    attached TEXT NULL DEFAULT NULL,
    timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    UNIQUE KEY unique_mission_user (mission_id, user_id),
    FOREIGN KEY (mission_id) REFERENCES missions(id),
    FOREIGN KEY (user_id) REFERENCES telegram_users(id)
);
`;

// Creazione della tabella platform_action_points con tutte le combinazioni e il campo points
const createPlatformActionPointsTable = `
CREATE TABLE IF NOT EXISTS platform_ranking_action_points (
    id INT AUTO_INCREMENT PRIMARY KEY,
    platform ENUM('instagram', 'facebook', 'telegram', 'tiktok') NOT NULL,
    action_type ENUM('enter', 'like', 'follow', 'comment', 'share', 'view') NOT NULL,
    points INT NOT NULL DEFAULT 0,
    UNIQUE KEY unique_platform_action (platform, action_type)
);
`;

const createPlatformActionPointsMinMaxTable = `
CREATE TABLE IF NOT EXISTS platform_action_points_min_max (
    id INT AUTO_INCREMENT PRIMARY KEY,
    platform ENUM('instagram', 'facebook', 'telegram', 'tiktok') NOT NULL,
    action_type ENUM('enter', 'like', 'follow', 'comment', 'share', 'view') NOT NULL,
    points_min INT NOT NULL DEFAULT 5,
    points_max INT NOT NULL DEFAULT 10,
    UNIQUE KEY unique_platform_action (platform, action_type)
);
`;

// Query per inserire tutte le combinazioni di platform e action_type con points = 0
const insertCombinations = `
INSERT IGNORE INTO platform_ranking_action_points (platform, action_type, points)
VALUES 
('instagram', 'enter', 5),
('instagram', 'like', 2),
('instagram', 'follow', 5),
('instagram', 'comment', 3),
('instagram', 'share', 3),
('instagram', 'view', 1),
('facebook', 'enter', 5),
('facebook', 'like', 2),
('facebook', 'follow', 5),
('facebook', 'comment', 3),
('facebook', 'share', 3),
('facebook', 'view', 1),
('telegram', 'enter', 5),
('telegram', 'like', 2),
('telegram', 'follow', 5),
('telegram', 'comment', 3),
('telegram', 'share', 3),
('telegram', 'view', 1),
('tiktok', 'enter', 5),
('tiktok', 'like', 2),
('tiktok', 'follow', 5),
('tiktok', 'comment', 3),
('tiktok', 'share', 3),
('tiktok', 'view', 1);
`;

// Query per inserire tutte le combinazioni di platform e action_type con points min 5 e max 10
const insertCombinationsMinMax = `
INSERT IGNORE INTO platform_action_points_min_max (platform, action_type)
VALUES 
('instagram', 'enter'),
('instagram', 'like'),
('instagram', 'follow'),
('instagram', 'comment'),
('instagram', 'share'),
('instagram', 'view'),
('facebook', 'enter'),
('facebook', 'like'),
('facebook', 'follow'),
('facebook', 'comment'),
('facebook', 'share'),
('facebook', 'view'),
('telegram', 'enter'),
('telegram', 'like'),
('telegram', 'follow'),
('telegram', 'comment'),
('telegram', 'share'),
('telegram', 'view'),
('tiktok', 'enter'),
('tiktok', 'like'),
('tiktok', 'follow'),
('tiktok', 'comment'),
('tiktok', 'share'),
('tiktok', 'view');
`;

const createTransactionsTable = `CREATE TABLE IF NOT EXISTS transactions (
    id INT AUTO_INCREMENT PRIMARY KEY,
    userId INT NOT NULL,
    uuid CHAR(36) NOT NULL UNIQUE,
    type ENUM('Crypto', 'GiftCard') NOT NULL,
    amount DECIMAL(10, 2) NOT NULL,
    status ENUM('in_progress', 'paid', 'cancelled') NOT NULL,
    type_gift TEXT NULL DEFAULT NULL,
    transaction_code TEXT NULL DEFAULT NULL,
    wallet TEXT NULL DEFAULT NULL,
    reason TEXT NULL DEFAULT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (userId) REFERENCES telegram_users(id)
);
`;

const createPackagesTable = `CREATE TABLE IF NOT EXISTS packages (
  id INT AUTO_INCREMENT PRIMARY KEY,
  description VARCHAR(255) NOT NULL UNIQUE,
  platform ENUM('instagram', 'facebook', 'telegram', 'tiktok') NULL DEFAULT NULL,
  max_user INT NULL DEFAULT NULL,
  action_type ENUM('enter', 'like', 'follow', 'comment', 'share', 'view')  NULL DEFAULT NULL,
  price DECIMAL(10, 2) NOT NULL
);
`;

const createPaymentTable = `
CREATE TABLE IF NOT EXISTS completed_payments (
  id INT AUTO_INCREMENT PRIMARY KEY,
  package_id INT NOT NULL,
  user_id INT NOT NULL,
  amount INT NOT NULL,
  status ENUM('completed') DEFAULT 'completed',
  typePoint BOOLEAN DEFAULT FALSE,
  mission_id INT NULL DEFAULT NULL,
  created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
  FOREIGN KEY (package_id) REFERENCES packages(id),
  FOREIGN KEY (mission_id) REFERENCES missions(id)
);`;
const insertPackages = `
INSERT IGNORE INTO packages (description, price, platform, max_user, action_type)
VALUES 
('1000 punti al costo di 4,99 € / fino a 200 interazioni', '4.99', NULL, NULL, NULL),
('5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%', '19.99', NULL, NULL, NULL),
('10 Follower 1,00 €', '1.00', 'instagram', 10, 'follow'),
('100 Follower 7,50 €', '7.50', 'instagram', 100,'follow'),
('10 Commenti 2,00 €', '2.00', 'instagram', 10,'comment'),
('20 Commenti 3,00 €', '3.00', 'instagram', 20,'comment'),
('10 Iscrizioni a Gruppo 1,00 €', '1.00', 'facebook', 10, 'enter'),
('50 Iscrizioni a Gruppo 4,00 €', '4.00', 'facebook', 50, 'enter'),
('10 Commenti 1,00 €', '1.00', 'facebook', 10,'comment'),
('20 Commenti 1,50 €', '1.50', 'facebook', 20,'comment'),
('10 iscrizioni a Canale/Gruppo 1,00 €', '1.00', 'telegram', 10, 'enter'),
('50 Iscrizioni a Canale/Gruppo 2,50 €', '2.50', 'telegram', 50, 'enter');
`;

const createNotificationsTable = `
CREATE TABLE IF NOT EXISTS notifications (
    id INT AUTO_INCREMENT PRIMARY KEY,
    user_id INT NOT NULL,
    text VARCHAR(255) NOT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (user_id) REFERENCES telegram_users(id)
);
`;

// Funzione per eseguire le query di creazione delle tabelle usando async/await
const createTables = async () => {
  const connection = await createConnection();

  try {
    await connection.query(createTelegramUsersTable);
    console.log("Table telegram_users created or already exists.");

    await connection.query(createNotificationsTable);
    console.log("Table notifications created or already exists.");

    await connection.query(createPackagesTable);
    console.log("Table packages created or already exists.");

    await connection.query(createPaymentTable);
    console.log("Table payment created or already exists.");

    await connection.query(createTransactionsTable);
    console.log("Table transactions created or already exists.");

    await connection.query(createMissionsTable);
    console.log("Table missions created or already exists.");

    await connection.query(createMissionStatusTable);
    console.log("Table mission_status created or already exists.");

    await connection.query(createPlatformActionPointsTable);
    console.log("Table platform_action_points created or already exists.");

    await connection.query(createPlatformActionPointsMinMaxTable);
    console.log(
      "Table platform_action_points_min_max created or already exists."
    );

    await connection.query(insertCombinations);
    console.log(
      "All platform-action combinations inserted with default points."
    );

    await connection.query(insertCombinationsMinMax);
    console.log(
      "All platform-action combinations min & max inserted with default points."
    );
    await connection.query(insertPackages);
    console.log("All package is added");
  } catch (err) {
    console.error("Error executing queries:", err);
  } finally {
    // Chiudi la connessione MySQL quando tutte le query sono completate
    await connection.end();
    console.log("Connection to MySQL closed.");
  }
};

// Esegui la creazione delle tabelle
createTables();
